@@ -25,6 +25,7 @@ Paul Licameli split from AudacityProject.cpp
 #include "DBConnection.h"
 #include "FileNames.h"
 #include "Project.h"
+#include "ProjectFileIOExtension.h"
 #include "ProjectHistory.h"
 #include "ProjectSerializer.h"
 #include "FileNames.h"
@@ -38,11 +39,14 @@ Paul Licameli split from AudacityProject.cpp
 #include "SentryHelper.h"
 #include "MemoryX.h"
 
+#include "ProjectFileIOExtension.h"
 #include "ProjectFormatExtensionsRegistry.h"
 
 #include "BufferedStreamReader.h"
 #include "FromChars.h"
 
+#include "sqlite/SQLiteUtils.h"
+
 // Don't change this unless the file format changes
 // in an irrevocable way
 #define AUDACITY_FILE_FORMAT_VERSION "1.3.0"
@@ -161,57 +165,6 @@ static const char *ProjectFileSchema =
    "  samples              BLOB"
    ");";
 
-// This singleton handles initialization/shutdown of the SQLite library.
-// It is needed because our local SQLite is built with SQLITE_OMIT_AUTOINIT
-// defined.
-//
-// It's safe to use even if a system version of SQLite is used that didn't
-// have SQLITE_OMIT_AUTOINIT defined.
-class SQLiteIniter
-{
-public:
-   SQLiteIniter()
-   {
-      // Enable URI filenames for all connections
-      mRc = sqlite3_config(SQLITE_CONFIG_URI, 1);
-      if (mRc == SQLITE_OK)
-      {
-         mRc = sqlite3_config(SQLITE_CONFIG_LOG, LogCallback, nullptr);
-         if (mRc == SQLITE_OK)
-         {
-            mRc = sqlite3_initialize();
-         }
-      }
-
-#ifdef NO_SHM
-      if (mRc == SQLITE_OK)
-      {
-         // Use the "unix-excl" VFS to make access to the DB exclusive.  This gets
-         // rid of the "<database name>-shm" shared memory file.
-         //
-         // Though it shouldn't, it doesn't matter if this fails.
-         auto vfs = sqlite3_vfs_find("unix-excl");
-         if (vfs)
-         {
-            sqlite3_vfs_register(vfs, 1);
-         }
-      }
-#endif
-   }
-   ~SQLiteIniter()
-   {
-      // This function must be called single-threaded only
-      // It returns a value, but there's nothing we can do with it
-      (void) sqlite3_shutdown();
-   }
-
-   static void LogCallback(void *WXUNUSED(arg), int code, const char *msg)
-   {
-      wxLogMessage("sqlite3 message: (%d) %s", code, msg);
-   }
-
-   int mRc;
-};
 
 class SQLiteBlobStream final
 {
@@ -348,7 +301,7 @@ public:
        // Reading 64k proved to be slower, (64k - 8) gives no measurable difference
        // to reading 32k.
        // Reading 4k is slower than reading 32k.
-       : BufferedStreamReader(32 * 1024) 
+       : BufferedStreamReader(32 * 1024)
        , mDB(db)
        , mSchema(schema)
        , mTable(table)
@@ -419,8 +372,16 @@ constexpr std::array<const char*, 2> BufferedProjectBlobStream::Columns;
 
 bool ProjectFileIO::InitializeSQL()
 {
-   static SQLiteIniter sqliteIniter;
-   return sqliteIniter.mRc == SQLITE_OK;
+   if (audacity::sqlite::Initialize().IsError())
+      return false;
+
+   audacity::sqlite::SetLogCallback(
+      [](int code, std::string_view message) {
+         // message is forwarded from SQLite, so it is null-terminated
+         wxLogMessage("SQLite error (%d): %s", code, message.data());
+      });
+
+   return true;
 }
 
 static const AudacityProject::AttachedObjects::RegisteredFactory sFileIOKey{
@@ -852,12 +813,25 @@ bool ProjectFileIO::InstallSchema(sqlite3 *db, const char *schema /* = "main" */
 // An SQLite function that takes a blockid and looks it up in a set of
 // blockids captured during project load.  If the blockid isn't found
 // in the set, it will be deleted.
+namespace
+{
+struct ContextData final
+{
+   const AudacityProject& project;
+   const BlockIDs& blockids;
+};
+}
+
 void ProjectFileIO::InSet(sqlite3_context *context, int argc, sqlite3_value **argv)
 {
-   BlockIDs *blockids = (BlockIDs *) sqlite3_user_data(context);
+   auto contextData = reinterpret_cast<ContextData*>(sqlite3_user_data(context));
    SampleBlockID blockid = sqlite3_value_int64(argv[0]);
 
-   sqlite3_result_int(context, blockids->find(blockid) != blockids->end());
+   sqlite3_result_int(
+      context,
+      contextData->blockids.find(blockid) != contextData->blockids.end() ||
+         ProjectFileIOExtensionRegistry::IsBlockLocked(
+            contextData->project, blockid));
 }
 
 bool ProjectFileIO::DeleteBlocks(const BlockIDs &blockids, bool complement)
@@ -865,6 +839,8 @@ bool ProjectFileIO::DeleteBlocks(const BlockIDs &blockids, bool complement)
    auto db = DB();
    int rc;
 
+   ContextData contextData{ mProject, blockids };
+
    auto cleanup = finally([&]
    {
       // Remove our function, whether it was successfully defined or not.
@@ -872,8 +848,7 @@ bool ProjectFileIO::DeleteBlocks(const BlockIDs &blockids, bool complement)
    });
 
    // Add the function used to verify each row's blockid against the set of active blockids
-   const void *p = &blockids;
-   rc = sqlite3_create_function(db, "inset", 1, SQLITE_UTF8 | SQLITE_DETERMINISTIC, const_cast<void*>(p), InSet, nullptr, nullptr);
+   rc = sqlite3_create_function(db, "inset", 1, SQLITE_UTF8 | SQLITE_DETERMINISTIC, &contextData, InSet, nullptr, nullptr);
    if (rc != SQLITE_OK)
    {
       ADD_EXCEPTION_CONTEXT("sqlite3.rc", std::to_string(rc));
@@ -1624,6 +1599,16 @@ void ProjectFileIO::SetFileName(const FilePath &fileName)
 {
    auto &project = mProject;
 
+   if (!fileName.empty() && fileName != mFileName)
+   {
+      BasicUI::CallAfter(
+         [wThis = weak_from_this()]
+         {
+            if (auto pThis = wThis.lock())
+               pThis->Publish(ProjectFileIOMessage::ProjectFilePathChange);
+         });
+   }
+
    if (!mFileName.empty())
    {
       ActiveProjects::Remove(mFileName);
@@ -2154,6 +2139,8 @@ bool ProjectFileIO::UpdateSaved(const TrackList *tracks)
       return false;
    }
 
+   ProjectFileIOExtensionRegistry::OnUpdateSaved(mProject, doc);
+
    return true;
 }
 
@@ -2338,16 +2325,14 @@ bool ProjectFileIO::SaveProject(
       // And make it the active project file 
       UseConnection(std::move(newConn), fileName);
    }
-   else
+
+   if (!UpdateSaved(nullptr))
    {
-      if ( !UpdateSaved( nullptr ) ) {
-         ShowError( {},
-            XO("Error Saving Project"),
-            FileException::WriteFailureMessage(fileName),
-            "Error:_Disk_full_or_not_writable"
-            );
-         return false;
-      }
+      ShowError(
+         {}, XO("Error Saving Project"),
+         FileException::WriteFailureMessage(fileName),
+         "Error:_Disk_full_or_not_writable");
+      return false;
    }
 
    // Reaching this point defines success and all the rest are no-fail
@@ -2379,13 +2364,13 @@ bool ProjectFileIO::OpenProject()
    return OpenConnection();
 }
 
-bool ProjectFileIO::CloseProject()
+void ProjectFileIO::CloseProject()
 {
    auto &currConn = CurrConn();
    if (!currConn)
    {
       wxLogDebug("Closing project with no database connection");
-      return true;
+      return;
    }
 
    // Save the filename since CloseConnection() will clear it
@@ -2407,8 +2392,6 @@ bool ProjectFileIO::CloseProject()
             RemoveProject(filename);
       }
    }
-
-   return true;
 }
 
 bool ProjectFileIO::ReopenProject()
@@ -2437,6 +2420,11 @@ bool ProjectFileIO::IsRecovered() const
    return mRecovered;
 }
 
+void ProjectFileIO::MarkTemporary()
+{
+   mTemporary = true;
+}
+
 wxLongLong ProjectFileIO::GetFreeDiskSpace() const
 {
    wxLongLong freeSpace;
@@ -2590,9 +2578,9 @@ int64_t ProjectFileIO::GetDiskUsage(DBConnection &conn, SampleBlockID blockid /*
    if (blockid == 0)
    {
       static const char* statement =
-R"(SELECT 
-	sum(length(blockid) + length(sampleformat) + 
-	length(summin) + length(summax) + length(sumrms) + 
+R"(SELECT
+	sum(length(blockid) + length(sampleformat) +
+	length(summin) + length(summax) + length(sumrms) +
 	length(summary256) + length(summary64k) +
 	length(samples))
 FROM sampleblocks;)";
@@ -2602,9 +2590,9 @@ FROM sampleblocks;)";
    else
    {
       static const char* statement =
-R"(SELECT 
-	length(blockid) + length(sampleformat) + 
-	length(summin) + length(summax) + length(sumrms) + 
+R"(SELECT
+	length(blockid) + length(sampleformat) +
+	length(summin) + length(summax) + length(sumrms) +
 	length(summary256) + length(summary64k) +
 	length(samples)
 FROM sampleblocks WHERE blockid = ?1;)";
