@@ -115,6 +115,8 @@ from there.  Audacity will look for a file called "Pause.png".
    DEFINE_IMAGE( bmpCopyDisabled, wxImage( 20, 20 ), wxT("CopyDisabled"));
    DEFINE_IMAGE( bmpPaste, wxImage( 20, 20 ), wxT("Paste"));
    DEFINE_IMAGE( bmpPasteDisabled, wxImage( 20, 20 ), wxT("PasteDisabled"));
+   DEFINE_IMAGE(bmpDelete, wxImage(20, 20), wxT("Delete"));
+   DEFINE_IMAGE(bmpDeleteDisabled, wxImage(20, 20), wxT("DeleteDisabled"));
 
    SET_THEME_FLAGS(  resFlagPaired | resFlagNewLine  );
    DEFINE_IMAGE( bmpTnStartOn, wxImage( 27, 27 ), wxT("TnStartOn"));
@@ -164,9 +166,7 @@ from there.  Audacity will look for a file called "Pause.png".
 
    SET_THEME_FLAGS(  resFlagNone  );
    DEFINE_IMAGE( bmpPlayPointer, wxImage( 20, 20 ), wxT("PlayPointer"));
-   DEFINE_IMAGE( bmpPlayPointerPinned, wxImage( 20, 20 ), wxT("PlayPointerPinned"));
    DEFINE_IMAGE( bmpRecordPointer, wxImage( 20, 20 ), wxT("RecordPointer"));
-   DEFINE_IMAGE( bmpRecordPointerPinned, wxImage( 20, 20 ), wxT("RecordPointerPinned"));
    DEFINE_IMAGE( bmpGrabberDropLoc, wxImage( 20, 20 ), wxT("GrabberDropLoc"));
    DEFINE_IMAGE( bmpSliderThumb, wxImage( 20, 20 ), wxT("SliderThumb"));
    DEFINE_IMAGE( bmpSliderThumbHilited, wxImage( 20, 20 ), wxT("SliderThumbHilited"));
@@ -252,16 +252,16 @@ from there.  Audacity will look for a file called "Pause.png".
    DEFINE_IMAGE( bmpHButtonDown, wxImage( 22, 22 ), wxT("HButtonDown"))
    DEFINE_IMAGE( bmpHButtonDisabled, wxImage( 22, 22 ), wxT("HButtonDisabled"))
 
+   SET_THEME_FLAGS(  resFlagNewLine  );
+   DEFINE_IMAGE(pitchUpIndicator, wxImage(12, 12), wxT("PitchUpIndicator"));
+   DEFINE_IMAGE(pitchDownIndicator, wxImage(12, 12), wxT("PitchDownIndicator"));
+   DEFINE_IMAGE(speedIndicator, wxImage(12, 12), wxT("speedIndicator"));
+
    //SET_THEME_FLAGS(  resFlagNewLine  );
 
-// DA: The logo with name xpm has a different width.
-#ifdef EXPERIMENTAL_DA
-#define LOGOWITHNAME_WIDTH 629
-#else
-#define LOGOWITHNAME_WIDTH 506
-#endif
 
-#define LOGOWITHNAME_HEIGHT 200
+#define LOGOWITHNAME_WIDTH 500
+#define LOGOWITHNAME_HEIGHT 134
 
    SET_THEME_FLAGS( resFlagNewLine );
    DEFINE_IMAGE( bmpAudacityLogo48x48, wxImage( 48, 48 ), wxT("AudacityLogo48x48"));
@@ -270,6 +270,11 @@ from there.  Audacity will look for a file called "Pause.png".
    DEFINE_IMAGE(bmpAnonymousUser, wxImage(20, 20), wxT("AnonymousUser"));
    DEFINE_IMAGE(bmpShareAudio, wxImage(20, 20), wxT("ShareAudio"));
 
+   SET_THEME_FLAGS(resFlagNewLine);
+   DEFINE_IMAGE(bmpCogwheel, wxImage(20, 20), wxT("Cogwheel"));
+   DEFINE_IMAGE(bmpCloud, wxImage(20, 20), wxT("Cloud"));
+   DEFINE_IMAGE(bmpCloudProgress, wxImage(20, 20), wxT("CloudProgress"));
+
 
    DEFINE_COLOUR( clrBlank,      wxColour( 64,  64,  64), wxT("Blank"));
    DEFINE_COLOUR( clrUnselected, wxColour( 30,  30,  30), wxT("Unselected"));
@@ -409,3 +414,10 @@ from there.  Audacity will look for a file called "Pause.png".
    DEFINE_COLOUR( clrDropHintHighlight, wxColour( 71, 140, 247 ), wxT("DropHintHighlight") );
    DEFINE_COLOUR( clrEffectListItemBackground, wxColour( 60, 60, 60 ), wxT("EffectListItemBackground"));
    DEFINE_COLOUR( clrEffectListItemBorder, wxColour( 204, 204, 204 ), wxT("EffectListItemBorder"));
+
+   DEFINE_COLOUR( clrBeatSeparatorPen, wxColour( 0xBE, 0xBE, 0xBE ), wxT("BeatSeparatorPen") );
+   DEFINE_COLOUR( clrBarSeparatorPen, wxColour( 0xBE, 0xBE, 0xBE ), wxT("BarSeparatorPen") );
+   DEFINE_COLOUR( clrBeatFillStrongBrush, wxColour( 0xD6, 0xD6, 0xD6 ), wxT("BeatFillStrongBrush") );
+   DEFINE_COLOUR( clrBeatFillWeakBrush, wxColour( 0xDC, 0xDC, 0xDC ), wxT("BeatFillWeakBrush") );
+   DEFINE_COLOUR( clrBeatFillStrongSelBrush, wxColour( 0xD6, 0xD6, 0xD6 ), wxT("BeatFillStrongSelBrush") );
+   DEFINE_COLOUR( clrBeatFillWeakSelBrush, wxColour( 0xDC, 0xDC, 0xDC ), wxT("BeatFillWeakSelBrush") );
