@@ -14,21 +14,29 @@
 
 #include "Internat.h"
 #include "TranslatableString.h"
+#include "WaveTrack.h"
 
 class WaveTrack;
 using ProgressReporter = std::function<void(double)>;
 
-namespace WaveTrackUtilities {
+namespace WaveTrackUtilities
+{
 
 //! Whether any clips, whose play regions intersect the interval, have non-unit
 //! stretch ratio
 WAVE_TRACK_API
-bool HasStretch(const WaveTrack &track, double t0, double t1);
+bool HasPitchOrSpeed(const WaveTrack& track, double t0, double t1);
 
 extern WAVE_TRACK_API const TranslatableString defaultStretchRenderingTitle;
 
-WAVE_TRACK_API void WithStretchRenderingProgress(
+WAVE_TRACK_API void WithClipRenderingProgress(
    std::function<void(const ProgressReporter&)> action,
    TranslatableString title = defaultStretchRenderingTitle,
-   TranslatableString message = XO("Rendering Time-Stretched Audio"));
-}
+   TranslatableString message = XO("Rendering Clip"));
+
+WAVE_TRACK_API bool SetClipStretchRatio(
+   const WaveTrack& track, WaveTrack::Interval& interval, double stretchRatio);
+
+WAVE_TRACK_API void
+ExpandClipTillNextOne(const WaveTrack& track, WaveTrack::Interval& interval);
+} // namespace WaveTrackUtilities
