@@ -1,109 +1,91 @@
-
-
 #include "../AdornedRulerPanel.h"
-#include "../AudioIO.h"
+#include "AudioIO.h"
+#include "BasicUI.h"
 #include "../CommonCommandFlags.h"
-#include "../SpectrumAnalyst.h"
 #include "Prefs.h"
 #include "Project.h"
-#include "../ProjectAudioIO.h"
+#include "ProjectAudioIO.h"
 #include "../ProjectAudioManager.h"
 #include "ProjectHistory.h"
 #include "ProjectRate.h"
-#include "../ProjectSelectionManager.h"
-#include "../ProjectSettings.h"
-#include "../ProjectWindow.h"
+#include "ProjectSnap.h"
 #include "../ProjectWindows.h"
 #include "../SelectUtilities.h"
-#include "../SyncLock.h"
+#include "SyncLock.h"
 #include "../TrackPanel.h"
-#include "../WaveTrack.h"
+#include "Viewport.h"
+#include "WaveClip.h"
+#include "WaveTrack.h"
 #include "../LabelTrack.h"
-#include "../commands/CommandContext.h"
-#include "../commands/CommandManager.h"
+#include "CommandContext.h"
+#include "MenuRegistry.h"
 #include "../toolbars/ControlToolBar.h"
 #include "../tracks/ui/SelectHandle.h"
 #include "../tracks/labeltrack/ui/LabelTrackView.h"
-#include "../tracks/playabletrack/wavetrack/ui/WaveTrackView.h"
-#include "../tracks/playabletrack/wavetrack/ui/WaveTrackViewConstants.h"
+#include "../tracks/playabletrack/wavetrack/ui/WaveChannelView.h"
 
 // private helper classes and functions
 namespace {
 
-void DoNextPeakFrequency(AudacityProject &project, bool up)
+constexpr auto GetWindowSize(double projectRate)
 {
-   auto &tracks = TrackList::Get( project );
-   auto &viewInfo = ViewInfo::Get( project );
-
-   // Find the first selected wave track that is in a spectrogram view.
-   const WaveTrack *pTrack {};
-   for ( auto wt : tracks.Selected< const WaveTrack >() ) {
-      const auto displays = WaveTrackView::Get( *wt ).GetDisplays();
-      bool hasSpectrum = (displays.end() != std::find(
-         displays.begin(), displays.end(),
-         WaveTrackSubView::Type{ WaveTrackViewConstants::Spectrum, {} }
-      ) );
-      if ( hasSpectrum ) {
-         pTrack = wt;
-         break;
-      }
-   }
-
-   if (pTrack) {
-      SpectrumAnalyst analyst;
-      SelectHandle::SnapCenterOnce(analyst, viewInfo, pTrack, up);
-      ProjectHistory::Get( project ).ModifyState(false);
-   }
+   return size_t(std::max(1.0, projectRate / 100));
 }
 
-double NearestZeroCrossing
-(AudacityProject &project, double t0)
+double NearestZeroCrossing(AudacityProject &project, double t0)
 {
    auto rate = ProjectRate::Get(project).GetRate();
    auto &tracks = TrackList::Get( project );
 
    // Window is 1/100th of a second.
-   auto windowSize = size_t(std::max(1.0, rate / 100));
+   auto windowSize = GetWindowSize(rate);
    Floats dist{ windowSize, true };
 
    int nTracks = 0;
-   for (auto one : tracks.Selected< const WaveTrack >()) {
+   for (auto one : tracks.Selected<const WaveTrack>()) {
+      const auto nChannels = one->NChannels();
       auto oneWindowSize = size_t(std::max(1.0, one->GetRate() / 100));
-      Floats oneDist{ oneWindowSize };
+      Floats buffer1{ oneWindowSize };
+      Floats buffer2{ oneWindowSize };
+      float *const buffers[]{ buffer1.get(), buffer2.get() };
       auto s = one->TimeToLongSamples(t0);
+
       // fillTwo to ensure that missing values are treated as 2, and hence do
       // not get used as zero crossings.
-      one->GetFloats(oneDist.get(),
-               s - (int)oneWindowSize/2, oneWindowSize, fillTwo);
+      one->GetFloats(0, nChannels, buffers,
+         s - (int)oneWindowSize/2, oneWindowSize, false, FillFormat::fillTwo);
 
 
-      // Looking for actual crossings.
-      double prev = 2.0;
-      for(size_t i=0; i<oneWindowSize; i++){
-         float fDist = fabs( oneDist[i]); // score is absolute value
-         if( prev * oneDist[i] > 0 ) // both same sign?  No good.
-            fDist = fDist + 0.4; // No good if same sign.
-         else if( prev > 0.0 )
-            fDist = fDist + 0.1; // medium penalty for downward crossing.
-         prev = oneDist[i];
-         oneDist[i] = fDist;
-      }
+      // Looking for actual crossings.  Update dist
+      for (size_t iChannel = 0; iChannel < nChannels; ++iChannel) {
+         const auto oneDist = buffers[iChannel];
+         double prev = 2.0;
+         for (size_t i = 0; i < oneWindowSize; ++i) {
+            float fDist = fabs(oneDist[i]); // score is absolute value
+            if (prev * oneDist[i] > 0) // both same sign?  No good.
+               fDist = fDist + 0.4; // No good if same sign.
+            else if (prev > 0.0)
+               fDist = fDist + 0.1; // medium penalty for downward crossing.
+            prev = oneDist[i];
+            oneDist[i] = fDist;
+         }
 
-      // TODO: The mixed rate zero crossing code is broken,
-      // if oneWindowSize > windowSize we'll miss out some
-      // samples - so they will still be zero, so we'll use them.
-      for(size_t i = 0; i < windowSize; i++) {
-         size_t j;
-         if (windowSize != oneWindowSize)
-            j = i * (oneWindowSize-1) / (windowSize-1);
-         else
-            j = i;
+         // TODO: The mixed rate zero crossing code is broken,
+         // if oneWindowSize > windowSize we'll miss out some
+         // samples - so they will still be zero, so we'll use them.
+         for (size_t i = 0; i < windowSize; i++) {
+            size_t j;
+            if (windowSize != oneWindowSize)
+               j = i * (oneWindowSize - 1) / (windowSize - 1);
+            else
+               j = i;
 
-         dist[i] += oneDist[j];
-         // Apply a small penalty for distance from the original endpoint
-         // We'll always prefer an upward  
-         dist[i] +=
-            0.1 * (abs(int(i) - int(windowSize/2))) / float(windowSize/2);
+            dist[i] += oneDist[j];
+            // Apply a small penalty for distance from the original endpoint
+            // We'll always prefer an upward
+            dist[i] +=
+               0.1 * (abs(int(i) - int(windowSize / 2))) / float(windowSize / 2);
+         }
       }
       nTracks++;
    }
@@ -111,7 +93,7 @@ double NearestZeroCrossing
    // Find minimum
    int argmin = 0;
    float min = 3.0;
-   for(size_t i=0; i<windowSize; i++) {
+   for (size_t i = 0; i < windowSize; ++i) {
       if (dist[i] < min) {
          argmin = i;
          min = dist[i];
@@ -119,13 +101,13 @@ double NearestZeroCrossing
    }
 
    // If we're worse than 0.2 on average, on one track, then no good.
-   if(( nTracks == 1 ) && ( min > (0.2*nTracks) ))
+   if ((nTracks == 1) && (min > (0.2 * nTracks)))
       return t0;
    // If we're worse than 0.6 on average, on multi-track, then no good.
-   if(( nTracks > 1 ) && ( min > (0.6*nTracks) ))
+   if ((nTracks > 1) && (min > (0.6 * nTracks)))
       return t0;
 
-   return t0 + (argmin - (int)windowSize/2) / rate;
+   return t0 + (argmin - (int)windowSize / 2) / rate;
 }
 
 // If this returns true, then there was a key up, and nothing more to do,
@@ -193,39 +175,31 @@ void SeekWhenAudioActive(double seekStep, wxLongLong &lastSelectionAdjustment)
 double GridMove
 (AudacityProject &project, double t, int minPix)
 {
-   auto &settings = ProjectSettings::Get(project);
-   auto rate = ProjectRate::Get(project).GetRate();
+   auto& projectSnap = ProjectSnap::Get(project);
    auto &viewInfo = ViewInfo::Get( project );
-   auto format = settings.GetSelectionFormat();
 
-   NumericConverter nc(NumericConverter::TIME, format, t, rate);
+   auto result = projectSnap.SingleStep(t, minPix >= 0).time;
 
-   // Try incrementing/decrementing the value; if we've moved far enough we're
-   // done
-   double result;
-   minPix >= 0 ? nc.Increment() : nc.Decrement();
-   result = nc.GetValue();
-   if (std::abs(viewInfo.TimeToPosition(result) - viewInfo.TimeToPosition(t))
-       >= abs(minPix))
-       return result;
+   if (
+      std::abs(viewInfo.TimeToPosition(result) - viewInfo.TimeToPosition(t)) >=
+      abs(minPix))
+      return result;
 
    // Otherwise, move minPix pixels, then snap to the time.
    result = viewInfo.OffsetTimeByPixels(t, minPix);
-   nc.SetValue(result);
-   result = nc.GetValue();
-   return result;
+   return projectSnap.SnapTime(result).time;
 }
 
 double OffsetTime
 (AudacityProject &project,
- double t, double offset, TimeUnit timeUnit, int snapToTime)
+ double t, double offset, TimeUnit timeUnit, SnapMode snapMode)
 {
    auto &viewInfo = ViewInfo::Get( project );
 
     if (timeUnit == TIME_UNIT_SECONDS)
         return t + offset; // snapping is currently ignored for non-pixel moves
 
-    if (snapToTime == SNAP_OFF)
+    if (snapMode == SnapMode::SNAP_OFF)
         return viewInfo.OffsetTimeByPixels(t, (int)offset);
 
     return GridMove(project, t, (int)offset);
@@ -235,30 +209,25 @@ double OffsetTime
 void MoveWhenAudioInactive
 (AudacityProject &project, double seekStep, TimeUnit timeUnit)
 {
-   auto &viewInfo = ViewInfo::Get( project );
-   auto &trackPanel = TrackPanel::Get( project );
-   auto &tracks = TrackList::Get( project );
-   auto &ruler = AdornedRulerPanel::Get( project );
-   const auto &settings = ProjectSettings::Get( project );
-   auto &window = ProjectWindow::Get( project );
+   auto &viewInfo = ViewInfo::Get(project);
+   auto &trackPanel = TrackPanel::Get(project);
+   auto &tracks = TrackList::Get(project);
+   auto &ruler = AdornedRulerPanel::Get(project);
+   const auto &settings = ProjectSnap::Get(project);
+   auto &viewport = Viewport::Get(project);
 
    // If TIME_UNIT_SECONDS, snap-to will be off.
-   int snapToTime = settings.GetSnapTo();
-   const double t0 = viewInfo.selectedRegion.t0();
-   const double end = std::max( 
-      tracks.GetEndTime(),
-      viewInfo.GetScreenEndTime());
+   auto snapMode = settings.GetSnapMode();
 
    // Move the cursor
    // Already in cursor mode?
-   if( viewInfo.selectedRegion.isPoint() )
+   if (viewInfo.selectedRegion.isPoint())
    {
-      double newT = OffsetTime(project,
-         t0, seekStep, timeUnit, snapToTime);
+      double newT = OffsetTime(
+         project, viewInfo.selectedRegion.t0(), seekStep, timeUnit, snapMode);
       // constrain.
       newT = std::max(0.0, newT);
-      newT = std::min(newT, end);
-      // Move 
+      // Move
       viewInfo.selectedRegion.setT0(
          newT,
          false); // do not swap selection boundaries
@@ -271,15 +240,28 @@ void MoveWhenAudioInactive
    else
    {
       // Transition to cursor mode.
-      if( seekStep < 0 )
+      constexpr auto maySwapBoundaries = false;
+      if (seekStep < 0)
+      {
+         if (snapMode != SnapMode::SNAP_OFF)
+            viewInfo.selectedRegion.setT0(
+               settings.SnapTime(viewInfo.selectedRegion.t0()).time,
+               maySwapBoundaries);
          viewInfo.selectedRegion.collapseToT0();
+      }
       else
+      {
+         if (snapMode != SnapMode::SNAP_OFF)
+            viewInfo.selectedRegion.setT1(
+               settings.SnapTime(viewInfo.selectedRegion.t1()).time,
+               maySwapBoundaries);
          viewInfo.selectedRegion.collapseToT1();
+      }
       trackPanel.Refresh(false);
    }
 
    // Make sure NEW position is in view
-   window.ScrollIntoView(viewInfo.selectedRegion.t1());
+   viewport.ScrollIntoView(viewInfo.selectedRegion.t1());
    return;
 }
 
@@ -287,30 +269,29 @@ void SeekWhenAudioInactive
 (AudacityProject &project, double seekStep, TimeUnit timeUnit,
 SelectionOperation operation)
 {
-   auto &viewInfo = ViewInfo::Get( project );
-   auto &tracks = TrackList::Get( project );
-   const auto &settings = ProjectSettings::Get( project );
-   auto &window = ProjectWindow::Get( project );
+   auto &viewInfo = ViewInfo::Get(project);
+   auto &tracks = TrackList::Get(project);
+   const auto &settings = ProjectSnap::Get(project);
+   auto &viewport = Viewport::Get(project);
 
-   if( operation == CURSOR_MOVE )
+   if (operation == CURSOR_MOVE)
    {
       MoveWhenAudioInactive( project, seekStep, timeUnit);
       return;
    }
 
-   int snapToTime = settings.GetSnapTo();
+   auto snapMode = settings.GetSnapMode();
    const double t0 = viewInfo.selectedRegion.t0();
    const double t1 = viewInfo.selectedRegion.t1();
-   const double end = std::max( 
-      tracks.GetEndTime(),
-      viewInfo.GetScreenEndTime());
+   const double end = std::max(
+      tracks.GetEndTime(), viewInfo.GetScreenEndTime());
 
    // Is it t0 or t1 moving?
    bool bMoveT0 = (operation == SELECTION_CONTRACT && seekStep > 0) ||
 	   (operation == SELECTION_EXTEND && seekStep < 0);
    // newT is where we want to move to
    double newT = OffsetTime( project,
-      bMoveT0 ? t0 : t1, seekStep, timeUnit, snapToTime);
+      bMoveT0 ? t0 : t1, seekStep, timeUnit, snapMode);
    // constrain to be in the track/screen limits.
    newT = std::max( 0.0, newT );
    newT = std::min( newT, end);
@@ -321,11 +302,11 @@ SelectionOperation operation)
    // Actually move
    if( bMoveT0 )
       viewInfo.selectedRegion.setT0( newT );
-   else 
+   else
       viewInfo.selectedRegion.setT1( newT );
 
    // Ensure it is visible
-   window.ScrollIntoView(newT);
+   viewport.ScrollIntoView(newT);
 }
 
 // Handle small cursor and play head movements
@@ -385,9 +366,9 @@ void DoCursorMove(
 
 void DoBoundaryMove(AudacityProject &project, int step, SeekInfo &info)
 {
-   auto &viewInfo = ViewInfo::Get( project );
-   auto &tracks = TrackList::Get( project );
-   auto &window = ProjectWindow::Get( project );
+   auto &viewInfo = ViewInfo::Get(project);
+   auto &tracks = TrackList::Get(project);
+   auto &viewport = Viewport::Get(project);
 
    // step is negative, then is moving left.  step positive, moving right.
    // Move the left/right selection boundary, to expand the selection
@@ -396,18 +377,15 @@ void DoBoundaryMove(AudacityProject &project, int step, SeekInfo &info)
    // holding the key down and should move faster.
    wxLongLong curtime = ::wxGetUTCTimeMillis();
    int pixels = step;
-   if( curtime - info.mLastSelectionAdjustment < 50 )
-   {
+   if (curtime - info.mLastSelectionAdjustment < 50)
       pixels *= 4;
-   }
    info.mLastSelectionAdjustment = curtime;
 
    // we used to have a parameter boundaryContract to say if expanding or
    // contracting.  it is no longer needed.
    bool bMoveT0 = (step < 0 );// ^ boundaryContract ;
 
-   if( ProjectAudioIO::Get( project ).IsAudioActive() )
-   {
+   if (ProjectAudioIO::Get(project).IsAudioActive()) {
       auto gAudioIO = AudioIO::Get();
       double indicator = gAudioIO->GetStreamTime();
       if( bMoveT0 )
@@ -421,9 +399,8 @@ void DoBoundaryMove(AudacityProject &project, int step, SeekInfo &info)
 
    const double t0 = viewInfo.selectedRegion.t0();
    const double t1 = viewInfo.selectedRegion.t1();
-   const double end = std::max( 
-      tracks.GetEndTime(),
-      viewInfo.GetScreenEndTime());
+   const double end = std::max(
+      tracks.GetEndTime(), viewInfo.GetScreenEndTime());
 
    double newT = viewInfo.OffsetTimeByPixels( bMoveT0 ? t0 : t1, pixels);
    // constrain to be in the track/screen limits.
@@ -436,11 +413,11 @@ void DoBoundaryMove(AudacityProject &project, int step, SeekInfo &info)
    // Actually move
    if( bMoveT0 )
       viewInfo.selectedRegion.setT0( newT );
-   else 
+   else
       viewInfo.selectedRegion.setT1( newT );
 
    // Ensure it is visible
-   window.ScrollIntoView(newT);
+   viewport.ScrollIntoView(newT);
 
    ProjectHistory::Get( project ).ModifyState(false);
 }
@@ -462,8 +439,8 @@ void OnSelectAll(const CommandContext &context)
 {
    auto& trackPanel = TrackPanel::Get(context.project);
    auto& tracks = TrackList::Get(context.project);
-   
-   for (auto lt : tracks.Selected< LabelTrack >()) {
+
+   for (auto lt : tracks.Selected<LabelTrack>()) {
       auto& view = LabelTrackView::Get(*lt);
       if (view.SelectAllText(context.project)) {
          trackPanel.Refresh(false);
@@ -474,7 +451,7 @@ void OnSelectAll(const CommandContext &context)
    //Presumably, there might be not more than one track
    //that expects text input
    for (auto wt : tracks.Any<WaveTrack>()) {
-      auto& view = WaveTrackView::Get(*wt);
+      auto& view = WaveChannelView::Get(*wt);
       if (view.SelectAllText(context.project)) {
          trackPanel.Refresh(false);
          return;
@@ -531,64 +508,62 @@ void OnSetRightSelection(const CommandContext &context)
 void OnSelectStartCursor(const CommandContext &context)
 {
    auto &project = context.project;
-   auto &tracks = TrackList::Get( project );
-   auto &selectedRegion = ViewInfo::Get( project ).selectedRegion;
+   auto &tracks = TrackList::Get(project);
+   auto &selectedRegion = ViewInfo::Get(project).selectedRegion;
 
    double kWayOverToRight = std::numeric_limits<double>::max();
 
    auto range = tracks.Selected();
-   if ( ! range )
+   if (!range)
       return;
 
-   double minOffset = range.min( &Track::GetStartTime );
+   double minOffset = range.min(&Track::GetStartTime);
 
    if( minOffset >=
        (kWayOverToRight * (1 - std::numeric_limits<double>::epsilon()) ))
       return;
 
    selectedRegion.setT0(minOffset);
-
-   ProjectHistory::Get( project ).ModifyState(false);
+   ProjectHistory::Get(project).ModifyState(false);
 }
 
 void OnSelectCursorEnd(const CommandContext &context)
 {
    auto &project = context.project;
-   auto &tracks = TrackList::Get( project );
-   auto &selectedRegion = ViewInfo::Get( project ).selectedRegion;
+   auto &tracks = TrackList::Get(project);
+   auto &selectedRegion = ViewInfo::Get(project).selectedRegion;
 
    double kWayOverToLeft = std::numeric_limits<double>::lowest();
 
    auto range = tracks.Selected();
-   if ( ! range )
+   if (!range)
       return;
 
-   double maxEndOffset = range.max( &Track::GetEndTime );
+   double maxEndOffset = range.max(&Track::GetEndTime);
 
    if( maxEndOffset <=
        (kWayOverToLeft * (1 - std::numeric_limits<double>::epsilon()) ))
       return;
 
    selectedRegion.setT1(maxEndOffset);
-
-   ProjectHistory::Get( project ).ModifyState(false);
+   ProjectHistory::Get(project).ModifyState(false);
 }
 
 void OnSelectTrackStartToEnd(const CommandContext &context)
 {
    auto &project = context.project;
-   auto &viewInfo = ViewInfo::Get( project );
-   auto &tracks = TrackList::Get( project );
+   auto &viewInfo = ViewInfo::Get(project);
+   auto &tracks = TrackList::Get(project);
 
    auto range = tracks.Selected();
-   double maxEndOffset = range.max( &Track::GetEndTime );
-   double minOffset = range.min( &Track::GetStartTime );
+   double maxEndOffset = range.max(&Track::GetEndTime);
+   double minOffset = range.min(&Track::GetStartTime);
 
    if( maxEndOffset < minOffset)
       return;
 
-   viewInfo.selectedRegion.setTimes( minOffset, maxEndOffset );
-   ProjectHistory::Get( project ).ModifyState(false);
+   viewInfo.selectedRegion.setTimes(minOffset, maxEndOffset);
+   ProjectHistory::Get(project).ModifyState(false);
 }
 
 // Handler state:
@@ -606,60 +581,18 @@ void OnSelectionRestore(const CommandContext &context)
 {
    auto &project = context.project;
    auto &selectedRegion = ViewInfo::Get( project ).selectedRegion;
-   auto &window = ProjectWindow::Get(project);
+   auto &viewport = Viewport::Get(project);
 
    if ((mRegionSave.t0() == 0.0) &&
        (mRegionSave.t1() == 0.0))
       return;
 
    selectedRegion = mRegionSave;
-   window.ScrollIntoView(selectedRegion.t0());
+   viewport.ScrollIntoView(selectedRegion.t0());
 
    ProjectHistory::Get( project ).ModifyState(false);
 }
 
-#ifdef EXPERIMENTAL_SPECTRAL_EDITING
-
-// Handler state:
-double mLastF0{ SelectedRegion::UndefinedFrequency };
-double mLastF1{ SelectedRegion::UndefinedFrequency };
-
-void OnToggleSpectralSelection(const CommandContext &context)
-{
-   auto &project = context.project;
-   auto &selectedRegion = ViewInfo::Get( project ).selectedRegion;
-
-   const double f0 = selectedRegion.f0();
-   const double f1 = selectedRegion.f1();
-   const bool haveSpectralSelection =
-   !(f0 == SelectedRegion::UndefinedFrequency &&
-     f1 == SelectedRegion::UndefinedFrequency);
-   if (haveSpectralSelection)
-   {
-      mLastF0 = f0;
-      mLastF1 = f1;
-      selectedRegion.setFrequencies
-      (SelectedRegion::UndefinedFrequency, SelectedRegion::UndefinedFrequency);
-   }
-   else
-      selectedRegion.setFrequencies(mLastF0, mLastF1);
-
-   ProjectHistory::Get( project ).ModifyState(false);
-}
-
-void OnNextHigherPeakFrequency(const CommandContext &context)
-{
-   auto &project = context.project;
-   DoNextPeakFrequency(project, true);
-}
-
-void OnNextLowerPeakFrequency(const CommandContext &context)
-{
-   auto &project = context.project;
-   DoNextPeakFrequency(project, false);
-}
-#endif
-
 // Handler state:
 bool mCursorPositionHasBeenStored{ false };
 double mCursorPositionStored{ 0.0 };
@@ -699,6 +632,40 @@ void OnZeroCrossing(const CommandContext &context)
 {
    auto &project = context.project;
    auto &selectedRegion = ViewInfo::Get( project ).selectedRegion;
+   const auto& tracks = TrackList::Get(project);
+
+   // Selecting precise sample indices across tracks that may have clips with
+   // various stretch ratios in itself is not possible. Even in single-track
+   // mode, we cannot know what the final waveform will look like until
+   // stretching is applied, making this operation futile. Hence we disallow
+   // it if any stretched clip is involved.
+   const auto projectRate = ProjectRate(project).GetRate();
+   const auto searchWindowDuration = GetWindowSize(projectRate) / projectRate;
+   const auto wouldSearchClipWithPitchOrSpeed =
+      [searchWindowDuration](const WaveTrack& track, double t) {
+         const auto clips = track.GetClipsIntersecting(
+            t - searchWindowDuration / 2, t + searchWindowDuration / 2);
+         return std::any_of(
+            clips.begin(), clips.end(),
+            [](const std::shared_ptr<const WaveClip>& clip) {
+               return clip->HasPitchOrSpeed();
+            });
+      };
+   const auto selected = tracks.Selected<const WaveTrack>();
+   if (std::any_of(
+          selected.begin(), selected.end(), [&](const WaveTrack* track) {
+             return wouldSearchClipWithPitchOrSpeed(
+                       *track, selectedRegion.t0()) ||
+                    wouldSearchClipWithPitchOrSpeed(
+                       *track, selectedRegion.t1());
+          }))
+   {
+      using namespace BasicUI;
+      ShowMessageBox(
+         XO("Zero-crossing search regions intersect stretched clip(s)."),
+         MessageBoxOptions {}.Caption(XO("Error")).IconStyle(Icon::Error));
+      return;
+   }
 
    const double t0 = NearestZeroCrossing(project, selectedRegion.t0());
    if (selectedRegion.isPoint())
@@ -716,34 +683,34 @@ void OnZeroCrossing(const CommandContext &context)
 void OnSnapToOff(const CommandContext &context)
 {
    auto &project = context.project;
-   ProjectSelectionManager::Get( project ).AS_SetSnapTo(SNAP_OFF);
+   ProjectSnap::Get(project).SetSnapMode(SnapMode::SNAP_OFF);
 }
 
 void OnSnapToNearest(const CommandContext &context)
 {
    auto &project = context.project;
-   ProjectSelectionManager::Get( project ).AS_SetSnapTo(SNAP_NEAREST);
+   ProjectSnap::Get( project ).SetSnapMode(SnapMode::SNAP_NEAREST);
 }
 
 void OnSnapToPrior(const CommandContext &context)
 {
    auto &project = context.project;
-   ProjectSelectionManager::Get( project ).AS_SetSnapTo(SNAP_PRIOR);
+   ProjectSnap::Get(project).SetSnapMode(SnapMode::SNAP_PRIOR);
 }
 
 void OnSelToStart(const CommandContext &context)
 {
    auto &project = context.project;
-   auto &window = ProjectWindow::Get( project );
-   window.Rewind(true);
+   auto &viewport = Viewport::Get(project);
+   viewport.ScrollToStart(true);
    ProjectHistory::Get( project ).ModifyState(false);
 }
 
 void OnSelToEnd(const CommandContext &context)
 {
    auto &project = context.project;
-   auto &window = ProjectWindow::Get( project );
-   window.SkipEnd(true);
+   auto &viewport = Viewport::Get(project);
+   viewport.ScrollToEnd(true);
    ProjectHistory::Get( project ).ModifyState(false);
 }
 
@@ -792,30 +759,30 @@ void OnCursorSelStart(const CommandContext &context)
 {
    auto &project = context.project;
    auto &selectedRegion = ViewInfo::Get( project ).selectedRegion;
-   auto &window = ProjectWindow::Get( project );
+   auto &viewport = Viewport::Get(project);
 
    selectedRegion.collapseToT0();
    ProjectHistory::Get( project ).ModifyState(false);
-   window.ScrollIntoView(selectedRegion.t0());
+   viewport.ScrollIntoView(selectedRegion.t0());
 }
 
 void OnCursorSelEnd(const CommandContext &context)
 {
    auto &project = context.project;
    auto &selectedRegion = ViewInfo::Get( project ).selectedRegion;
-   auto &window = ProjectWindow::Get( project );
+   auto &viewport = Viewport::Get(project);
 
    selectedRegion.collapseToT1();
    ProjectHistory::Get( project ).ModifyState(false);
-   window.ScrollIntoView(selectedRegion.t1());
+   viewport.ScrollIntoView(selectedRegion.t1());
 }
 
 void OnCursorTrackStart(const CommandContext &context)
 {
    auto &project = context.project;
-   auto &tracks = TrackList::Get( project );
-   auto &selectedRegion = ViewInfo::Get( project ).selectedRegion;
-   auto &window = ProjectWindow::Get( project );
+   auto &tracks = TrackList::Get(project);
+   auto &selectedRegion = ViewInfo::Get(project).selectedRegion;
+   auto &viewport = Viewport::Get(project);
 
    double kWayOverToRight = std::numeric_limits<double>::max();
 
@@ -825,23 +792,23 @@ void OnCursorTrackStart(const CommandContext &context)
       return;
 
    // Range is surely nonempty now
-   auto minOffset = std::max( 0.0, trackRange.min( &Track::GetOffset ) );
+   auto minOffset = std::max(0.0, trackRange.min(&Track::GetStartTime));
 
    if( minOffset >=
        (kWayOverToRight * (1 - std::numeric_limits<double>::epsilon()) ))
       return;
 
    selectedRegion.setTimes(minOffset, minOffset);
-   ProjectHistory::Get( project ).ModifyState(false);
-   window.ScrollIntoView(selectedRegion.t0());
+   ProjectHistory::Get(project).ModifyState(false);
+   viewport.ScrollIntoView(selectedRegion.t0());
 }
 
 void OnCursorTrackEnd(const CommandContext &context)
 {
    auto &project = context.project;
-   auto &tracks = TrackList::Get( project );
-   auto &selectedRegion = ViewInfo::Get( project ).selectedRegion;
-   auto &window = ProjectWindow::Get( project );
+   auto &tracks = TrackList::Get(project);
+   auto &selectedRegion = ViewInfo::Get(project).selectedRegion;
+   auto &viewport = Viewport::Get(project);
 
    double kWayOverToLeft = std::numeric_limits<double>::lowest();
 
@@ -851,15 +818,15 @@ void OnCursorTrackEnd(const CommandContext &context)
       return;
 
    // Range is surely nonempty now
-   auto maxEndOffset = trackRange.max( &Track::GetEndTime );
+   auto maxEndOffset = trackRange.max(&Track::GetEndTime);
 
    if( maxEndOffset <
        (kWayOverToLeft * (1 - std::numeric_limits<double>::epsilon()) ))
       return;
 
    selectedRegion.setTimes(maxEndOffset, maxEndOffset);
-   ProjectHistory::Get( project ).ModifyState(false);
-   window.ScrollIntoView(selectedRegion.t1());
+   ProjectHistory::Get(project).ModifyState(false);
+   viewport.ScrollIntoView(selectedRegion.t1());
 }
 
 void OnSkipStart(const CommandContext &context)
@@ -962,8 +929,8 @@ Handler()
 {
    UpdatePrefs();
 }
-Handler( const Handler & ) PROHIBITED;
-Handler &operator=( const Handler & ) PROHIBITED;
+Handler( const Handler & ) = delete;
+Handler &operator=( const Handler & ) = delete;
 
 }; // struct Handler
 
@@ -984,11 +951,10 @@ static CommandHandlerObject &findCommandHandler(AudacityProject &project) {
 #define FN(X) (& SelectActions::Handler :: X)
 
 namespace {
-using namespace MenuTable;
-BaseItemSharedPtr SelectMenu()
+using namespace MenuRegistry;
+auto SelectMenu()
 {
-   using Options = CommandManager::Options;
-   static BaseItemSharedPtr menu{
+   static auto menu = std::shared_ptr{
    ( FinderScope{ findCommandHandler },
    /* i18n-hint: (verb) It's an item on a menu. */
    Menu( wxT("Select"), XXO("&Select"),
@@ -1006,15 +972,11 @@ BaseItemSharedPtr SelectMenu()
             Command( wxT("SelAllTracks"), XXO("In All &Tracks"),
                FN(OnSelectAllTracks),
                TracksExistFlag(),
-               wxT("Ctrl+Shift+K") )
-
-   #ifdef EXPERIMENTAL_SYNC_LOCK
-            ,
+               wxT("Ctrl+Shift+K") ),
             Command( wxT("SelSyncLockTracks"), XXO("In All &Sync-Locked Tracks"),
                FN(OnSelectSyncLockSel),
                EditableTracksSelectedFlag() | IsSyncLockedFlag(),
                Options{ wxT("Ctrl+Shift+Y"), XO("Select Sync-Locked") } )
-   #endif
          ),
 
          //////////////////////////////////////////////////////////////////////////
@@ -1050,23 +1012,10 @@ BaseItemSharedPtr SelectMenu()
                Command( wxT("SelRestore"), XXO("Retrieve Selectio&n"),
                   FN(OnSelectionRestore), TracksExistFlag() )
             )
-         ),
+         )
 
          //////////////////////////////////////////////////////////////////////////
 
-   #ifdef EXPERIMENTAL_SPECTRAL_EDITING
-         Menu( wxT("Spectral"), XXO("S&pectral"),
-            Command( wxT("ToggleSpectralSelection"),
-               XXO("To&ggle Spectral Selection"), FN(OnToggleSpectralSelection),
-               TracksExistFlag(), wxT("Q") ),
-            Command( wxT("NextHigherPeakFrequency"),
-               XXO("Next &Higher Peak Frequency"), FN(OnNextHigherPeakFrequency),
-               TracksExistFlag() ),
-            Command( wxT("NextLowerPeakFrequency"),
-               XXO("Next &Lower Peak Frequency"), FN(OnNextLowerPeakFrequency),
-               TracksExistFlag() )
-         )
-   #endif
       ),
 
       Section( "",
@@ -1091,15 +1040,11 @@ BaseItemSharedPtr SelectMenu()
    return menu;
 }
 
-AttachedItem sAttachment1{
-   wxT(""),
-   Shared( SelectMenu() )
-};
+AttachedItem sAttachment1{ Indirect(SelectMenu()) };
 
-BaseItemSharedPtr ExtraSelectionMenu()
+auto ExtraSelectionMenu()
 {
-   using Options = CommandManager::Options;
-   static BaseItemSharedPtr menu{
+   static auto menu = std::shared_ptr{
    ( FinderScope{ findCommandHandler },
    Menu( wxT("Select"), XXO("&Selection"),
       Command( wxT("SnapToOff"), XXO("Snap-To &Off"), FN(OnSnapToOff),
@@ -1138,16 +1083,14 @@ BaseItemSharedPtr ExtraSelectionMenu()
    return menu;
 }
 
-AttachedItem sAttachment2{
-   wxT("Optional/Extra/Part1"),
-   Shared( ExtraSelectionMenu() )
+AttachedItem sAttachment2{ Indirect(ExtraSelectionMenu()),
+   wxT("Optional/Extra/Part1")
 };
 }
 
 namespace {
-BaseItemSharedPtr CursorMenu()
+auto CursorMenu()
 {
-   using Options = CommandManager::Options;
    static const auto CanStopFlags = AudioIONotBusyFlag() | CanStopAudioStreamFlag();
 
    // JKC: ANSWER-ME: How is 'cursor to' different to 'Skip To' and how is it
@@ -1155,7 +1098,7 @@ BaseItemSharedPtr CursorMenu()
    // GA: 'Skip to' moves the viewpoint to center of the track and preserves the
    // selection. 'Cursor to' does neither. 'Center at' might describe it better
    // than 'Skip'.
-   static BaseItemSharedPtr menu{
+   static auto menu = std::shared_ptr{
    ( FinderScope{ findCommandHandler },
    Menu( wxT("Cursor"), XXO("&Cursor to"),
       Command( wxT("CursSelStart"), XXO("Selection Star&t"),
@@ -1187,15 +1130,13 @@ BaseItemSharedPtr CursorMenu()
    return menu;
 }
 
-AttachedItem sAttachment0{
-   wxT("Transport/Basic"),
-   Shared( CursorMenu() )
+AttachedItem sAttachment0{ Indirect(CursorMenu()),
+   wxT("Transport/Basic")
 };
 
-BaseItemSharedPtr ExtraCursorMenu()
+auto ExtraCursorMenu()
 {
-   using Options = CommandManager::Options;
-   static BaseItemSharedPtr menu{
+   static auto menu = std::shared_ptr{
    ( FinderScope{ findCommandHandler },
    Menu( wxT("Cursor"), XXO("&Cursor"),
       Command( wxT("CursorLeft"), XXO("Cursor &Left"), FN(OnCursorLeft),
@@ -1220,15 +1161,13 @@ BaseItemSharedPtr ExtraCursorMenu()
    return menu;
 }
 
-AttachedItem sAttachment4{
-   wxT("Optional/Extra/Part2"),
-   Shared( ExtraCursorMenu() )
+AttachedItem sAttachment4{ Indirect(ExtraCursorMenu()),
+   wxT("Optional/Extra/Part2")
 };
 
-BaseItemSharedPtr ExtraSeekMenu()
+auto ExtraSeekMenu()
 {
-   using Options = CommandManager::Options;
-   static BaseItemSharedPtr menu{
+   static auto menu = std::shared_ptr{
    ( FinderScope{ findCommandHandler },
    Menu( wxT("Seek"), XXO("See&k"),
       Command( wxT("SeekLeftShort"), XXO("Short Seek &Left During Playback"),
@@ -1248,9 +1187,8 @@ BaseItemSharedPtr ExtraSeekMenu()
    return menu;
 }
 
-AttachedItem sAttachment5{
-   wxT("Optional/Extra/Part1"),
-   Shared( ExtraSeekMenu() )
+AttachedItem sAttachment5{ Indirect(ExtraSeekMenu()),
+   wxT("Optional/Extra/Part1")
 };
 
 }
