@@ -1,41 +1,30 @@
-
-
-#include "../AdornedRulerPanel.h"
-#include "../AudioIO.h"
+#include "AudioIO.h"
 #include "../CommonCommandFlags.h"
 #include "DeviceManager.h"
 #include "../LabelTrack.h"
-#include "../Menus.h"
 #include "Prefs.h"
 #include "Project.h"
-#include "../ProjectAudioIO.h"
+#include "ProjectAudioIO.h"
 #include "../ProjectAudioManager.h"
-#include "../ProjectFileIO.h"
 #include "ProjectHistory.h"
-#include "../ProjectManager.h"
-#include "ProjectRate.h"
-#include "../ProjectSettings.h"
 #include "../ProjectWindows.h"
-#include "../ProjectWindow.h"
 #include "../SelectUtilities.h"
 #include "../SoundActivatedRecord.h"
-#include "../TimerRecordDialog.h"
-#include "../TrackPanelAx.h"
+#include "TrackFocus.h"
 #include "../TrackPanel.h"
 #include "../TransportUtilities.h"
 #include "UndoManager.h"
-#include "../WaveClip.h"
 #include "../prefs/RecordingPrefs.h"
 #include "../prefs/TracksPrefs.h"
-#include "../WaveTrack.h"
+#include "WaveTrack.h"
 #include "ViewInfo.h"
-#include "../commands/CommandContext.h"
-#include "../commands/CommandManager.h"
+#include "Viewport.h"
+#include "CommandContext.h"
 #include "../toolbars/ControlToolBar.h"
-#include "../toolbars/TranscriptionToolBar.h"
-#include "../widgets/AudacityMessageBox.h"
+#include "../toolbars/ToolManager.h"
+#include "AudacityMessageBox.h"
 #include "BasicUI.h"
-#include "../widgets/ProgressDialog.h"
+#include "ProgressDialog.h"
 
 #include <thread>
 #include <float.h>
@@ -82,7 +71,7 @@ void DoMoveToLabel(AudacityProject &project, bool next)
 {
    auto &tracks = TrackList::Get( project );
    auto &trackFocus = TrackFocus::Get( project );
-   auto &window = ProjectWindow::Get( project );
+   auto &viewport = Viewport::Get(project);
    auto &projectAudioManager = ProjectAudioManager::Get(project);
 
    // Find the number of label tracks, and ptr to last track found
@@ -118,13 +107,13 @@ void DoMoveToLabel(AudacityProject &project, bool next)
          if (ProjectAudioIO::Get( project ).IsAudioActive()) {
             TransportUtilities::DoStopPlaying(project);
             selectedRegion = label->selectedRegion;
-            window.RedrawProject();
+            viewport.Redraw();
             TransportUtilities::DoStartPlaying(project, newDefault);
          }
          else {
             selectedRegion = label->selectedRegion;
-            window.ScrollIntoView(selectedRegion.t0());
-            window.RedrawProject();
+            viewport.ScrollIntoView(selectedRegion.t0());
+            viewport.Redraw();
          }
          /* i18n-hint:
             String is replaced by the name of a label,
@@ -158,9 +147,7 @@ static const auto SetLoopOutTitle = XXO("Set Loop &Out");
 
 // Menu handler functions
 
-namespace TransportActions {
-
-struct Handler : CommandHandlerObject {
+namespace {
 
 // This plays (once, with fixed bounds) OR Stops audio.  It's a toggle.
 // The default binding for Shift+SPACE.
@@ -209,145 +196,6 @@ void OnRecord2ndChoice(const CommandContext &context)
    TransportUtilities::RecordAndWait(context, true);
 }
 
-void OnTimerRecord(const CommandContext &context)
-{
-   auto &project = context.project;
-   const auto &settings = ProjectSettings::Get( project );
-   auto &undoManager = UndoManager::Get( project );
-   auto &window = ProjectWindow::Get( project );
-
-   // MY: Due to improvements in how Timer Recording saves and/or exports
-   // it is now safer to disable Timer Recording when there is more than
-   // one open project.
-   if (AllProjects{}.size() > 1) {
-      AudacityMessageBox(
-         XO(
-"Timer Recording cannot be used with more than one open project.\n\nPlease close any additional projects and try again."),
-         XO("Timer Recording"),
-         wxICON_INFORMATION | wxOK);
-      return;
-   }
-
-   // MY: If the project has unsaved changes then we no longer allow access
-   // to Timer Recording.  This decision has been taken as the safest approach
-   // preventing issues surrounding "dirty" projects when Automatic Save/Export
-   // is used in Timer Recording.
-   if ((undoManager.UnsavedChanges()) &&
-       (TrackList::Get( project ).Any() || settings.EmptyCanBeDirty())) {
-      AudacityMessageBox(
-         XO(
-"Timer Recording cannot be used while you have unsaved changes.\n\nPlease save or close this project and try again."),
-         XO("Timer Recording"),
-         wxICON_INFORMATION | wxOK);
-      return;
-   }
-
-   // We check the selected tracks to see if there is enough of them to accommodate
-   // all input channels and all of them have the same sampling rate.
-   // Those checks will be later performed by recording function anyway,
-   // but we want to warn the user about potential problems from the very start.
-   const auto selectedTracks{ GetPropertiesOfSelected(project) };
-   const int rateOfSelected{ selectedTracks.rateOfSelected };
-   const int numberOfSelected{ selectedTracks.numberOfSelected };
-   const bool allSameRate{ selectedTracks.allSameRate };
-
-   if (!allSameRate) {
-      AudacityMessageBox(XO("The tracks selected "
-         "for recording must all have the same sampling rate"),
-         XO("Mismatched Sampling Rates"),
-         wxICON_ERROR | wxCENTRE);
-
-      return;
-   }
-
-   const auto existingTracks{ ProjectAudioManager::ChooseExistingRecordingTracks(project, true, rateOfSelected) };
-   if (existingTracks.empty()) {
-      if (numberOfSelected > 0 && rateOfSelected !=
-          ProjectRate::Get(project).GetRate()) {
-         AudacityMessageBox(XO(
-            "Too few tracks are selected for recording at this sample rate.\n"
-            "(Audacity requires two channels at the same sample rate for\n"
-            "each stereo track)"),
-            XO("Too Few Compatible Tracks Selected"),
-            wxICON_ERROR | wxCENTRE);
-
-         return;
-      }
-   }
-   
-   // We use this variable to display "Current Project" in the Timer Recording
-   // save project field
-   bool bProjectSaved = !ProjectFileIO::Get( project ).IsModified();
-
-   //we break the prompting and waiting dialogs into two sections
-   //because they both give the user a chance to click cancel
-   //and therefore remove the newly inserted track.
-
-   TimerRecordDialog dialog(
-      &window, project, bProjectSaved); /* parent, project, project saved? */
-   int modalResult = dialog.ShowModal();
-   if (modalResult == wxID_CANCEL)
-   {
-      // Cancelled before recording - don't need to do anything.
-   }
-   else
-   {
-      // Bug #2382
-      // Allow recording to start at current cursor position.
-      #if 0
-      // Timer Record should not record into a selection.
-      bool bPreferNewTrack;
-      gPrefs->Read("/GUI/PreferNewTrackRecord",&bPreferNewTrack, false);
-      if (bPreferNewTrack) {
-         window.Rewind(false);
-      } else {
-         window.SkipEnd(false);
-      }
-      #endif
-
-      int iTimerRecordingOutcome = dialog.RunWaitDialog();
-      switch (iTimerRecordingOutcome) {
-      case POST_TIMER_RECORD_CANCEL_WAIT:
-         // Canceled on the wait dialog
-         ProjectHistory::Get( project ).RollbackState();
-         break;
-      case POST_TIMER_RECORD_CANCEL:
-         // RunWaitDialog() shows the "wait for start" as well as "recording"
-         // dialog if it returned POST_TIMER_RECORD_CANCEL it means the user
-         // cancelled while the recording, so throw out the fresh track.
-         // However, we can't undo it here because the PushState() is called in TrackPanel::OnTimer(),
-         // which is blocked by this function.
-         // so instead we mark a flag to undo it there.
-         ProjectAudioManager::Get( project ).SetTimerRecordCancelled();
-         break;
-      case POST_TIMER_RECORD_NOTHING:
-         // No action required
-         break;
-      case POST_TIMER_RECORD_CLOSE:
-         wxTheApp->CallAfter( []{
-            // Simulate the application Exit menu item
-            wxCommandEvent evt{ wxEVT_MENU, wxID_EXIT };
-            wxTheApp->AddPendingEvent( evt );
-         } );
-         ProjectManager::Get(project).SetSkipSavePrompt(true);
-         break;
-
-#ifdef __WINDOWS__
-      case POST_TIMER_RECORD_RESTART:
-         // Restart System
-         ProjectManager::Get(project).SetSkipSavePrompt(true);
-         system("shutdown /r /f /t 30");
-         break;
-      case POST_TIMER_RECORD_SHUTDOWN:
-         // Shutdown System
-         ProjectManager::Get(project).SetSkipSavePrompt(true);
-         system("shutdown /s /f /t 30");
-         break;
-#endif
-      }
-   }
-}
-
 #ifdef EXPERIMENTAL_PUNCH_AND_ROLL
 void OnPunchAndRoll(const CommandContext &context)
 {
@@ -413,22 +261,35 @@ void OnPunchAndRoll(const CommandContext &context)
    bool error = (t1 == 0.0);
 
    double newt1 = t1;
+   using Iterator =
+      ChannelGroup::IntervalIterator<const WideChannelGroupInterval>;
    for (const auto &wt : tracks) {
-      sampleCount testSample(floor(t1 * wt->GetRate()));
-      auto clip = wt->GetClipAtSample(testSample);
-      if (!clip)
+      auto rate = wt->GetRate();
+      sampleCount testSample(floor(t1 * rate));
+      const auto intervals = as_const(*wt).Intervals();
+      auto pred = [rate](sampleCount testSample){ return
+         [rate, testSample](const auto &pInterval){
+            auto start = floor(pInterval->Start() * rate + 0.5);
+            auto end = floor(pInterval->End() * rate + 0.5);
+            auto ts = testSample.as_double();
+            return ts >= start && ts < end;
+         };
+      };
+      auto begin = intervals.begin(), end = intervals.end(),
+         iter = std::find_if(begin, end, pred(testSample));
+      if (iter == end)
          // Bug 1890 (an enhancement request)
          // Try again, a little to the left.
          // Subtract 10 to allow a selection exactly at or slightly after the
          // end time
-         clip = wt->GetClipAtSample(testSample - 10);
-      if (!clip)
+         iter = std::find_if(begin, end, pred(testSample - 10));
+      if (iter == end)
          error = true;
       else {
          // May adjust t1 left
          // Let's ignore the possibility of a clip even shorter than the
          // crossfade duration!
-         newt1 = std::min(newt1, clip->GetPlayEndTime() - crossFadeDuration);
+         newt1 = std::min(newt1, (*iter).get()->End() - crossFadeDuration);
       }
    }
 
@@ -445,41 +306,48 @@ void OnPunchAndRoll(const CommandContext &context)
       const auto duration =
          std::max(0.0, std::min(crossFadeDuration, endTime - t1));
       const size_t getLen = floor(duration * wt->GetRate());
-      std::vector<float> data(getLen);
       if (getLen > 0) {
-         float *const samples = data.data();
+         // TODO more-than-two-channels
+         const auto nChannels = std::min<size_t>(2, wt->NChannels());
+         crossfadeData.resize(nChannels);
+         float *buffers[2]{};
+         for (size_t ii = 0; ii < nChannels; ++ii) {
+            auto &data = crossfadeData[ii];
+            data.resize(getLen);
+            buffers[ii] = data.data();
+         }
          const sampleCount pos = wt->TimeToLongSamples(t1);
-         wt->GetFloats(samples, pos, getLen);
+         if (!wt->GetFloats(0, nChannels, buffers, pos, getLen))
+            // TODO error message
+            return;
       }
-      crossfadeData.push_back(std::move(data));
    }
 
    // Change tracks only after passing the error checks above
-   for (const auto &wt : tracks) {
+   for (const auto &wt : tracks)
       wt->Clear(t1, wt->GetEndTime());
-   }
 
    // Choose the tracks for playback.
-   TransportTracks transportTracks;
+   TransportSequences transportTracks;
    const auto duplex = ProjectAudioManager::UseDuplex();
    if (duplex)
       // play all
-      transportTracks =
-         ProjectAudioManager::GetAllPlaybackTracks(
-            TrackList::Get( project ), false, true);
+      transportTracks = MakeTransportTracks(
+         TrackList::Get( project ), false, true);
    else
       // play recording tracks only
-      std::copy(tracks.begin(), tracks.end(),
-         std::back_inserter(transportTracks.playbackTracks));
+      for (auto &pTrack : tracks)
+         transportTracks.playbackSequences.push_back(pTrack);
       
    // Unlike with the usual recording, a track may be chosen both for playback
    // and recording.
-   transportTracks.captureTracks = std::move(tracks);
+   std::copy(tracks.begin(), tracks.end(),
+      back_inserter(transportTracks.captureSequences));
 
    // Try to start recording
-   auto options = DefaultPlayOptions( project );
+   auto options = ProjectAudioIO::GetDefaultOptions(project);
    options.rate = rateOfSelected;
-   options.preRoll = std::max(0L,
+   options.preRoll = std::max(0.0,
       gPrefs->Read(AUDIO_PRE_ROLL_KEY, DEFAULT_PRE_ROLL_SECONDS));
    options.pCrossfadeData = &crossfadeData;
    bool success = ProjectAudioManager::Get( project ).DoRecord(project,
@@ -548,29 +416,18 @@ void OnSoundActivated(const CommandContext &context)
 
 void OnToggleSoundActivated(const CommandContext &WXUNUSED(context) )
 {
-   bool pause;
-   gPrefs->Read(wxT("/AudioIO/SoundActivatedRecord"), &pause, false);
-   gPrefs->Write(wxT("/AudioIO/SoundActivatedRecord"), !pause);
+   SoundActivatedRecord.Toggle();
    gPrefs->Flush();
-   MenuManager::ModifyAllProjectToolbarMenus();
-}
-
-void OnTogglePinnedHead(const CommandContext &context)
-{
-   AdornedRulerPanel::Get( context.project ).TogglePinnedHead();
+   ToolManager::ModifyAllProjectToolbarMenus();
 }
 
 void OnTogglePlayRecording(const CommandContext &WXUNUSED(context) )
 {
    bool Duplex;
-#ifdef EXPERIMENTAL_DA
-   gPrefs->Read(wxT("/AudioIO/Duplex"), &Duplex, false);
-#else
    gPrefs->Read(wxT("/AudioIO/Duplex"), &Duplex, true);
-#endif
    gPrefs->Write(wxT("/AudioIO/Duplex"), !Duplex);
    gPrefs->Flush();
-   MenuManager::ModifyAllProjectToolbarMenus();
+   ToolManager::ModifyAllProjectToolbarMenus();
 }
 
 void OnToggleSWPlaythrough(const CommandContext &WXUNUSED(context) )
@@ -579,7 +436,7 @@ void OnToggleSWPlaythrough(const CommandContext &WXUNUSED(context) )
    gPrefs->Read(wxT("/AudioIO/SWPlaythrough"), &SWPlaythrough, false);
    gPrefs->Write(wxT("/AudioIO/SWPlaythrough"), !SWPlaythrough);
    gPrefs->Flush();
-   MenuManager::ModifyAllProjectToolbarMenus();
+   ToolManager::ModifyAllProjectToolbarMenus();
 }
 
 #ifdef EXPERIMENTAL_AUTOMATED_INPUT_LEVEL_ADJUSTMENT
@@ -591,7 +448,7 @@ void OnToggleAutomatedInputLevelAdjustment(
       wxT("/AudioIO/AutomatedInputLevelAdjustment"), &AVEnabled, false);
    gPrefs->Write(wxT("/AudioIO/AutomatedInputLevelAdjustment"), !AVEnabled);
    gPrefs->Flush();
-   MenuManager::ModifyAllProjectToolbarMenus();
+   ToolManager::ModifyAllProjectToolbarMenus();
 }
 #endif
 
@@ -607,7 +464,7 @@ void OnPlayOneSecond(const CommandContext &context)
       return;
 
    auto &trackPanel = TrackPanel::Get( project );
-   auto options = DefaultPlayOptions( project );
+   auto options = ProjectAudioIO::GetDefaultOptions(project);
 
    double pos = trackPanel.GetMostRecentXPos();
    TransportUtilities::PlayPlayRegionAndWait(
@@ -659,7 +516,7 @@ void OnPlayToSelection(const CommandContext &context)
    // only when playing a short region, less than or equal to a second.
 //   mLastPlayMode = ((t1-t0) > 1.0) ? normalPlay : oneSecondPlay;
 
-   auto playOptions = DefaultPlayOptions( project );
+   auto playOptions = ProjectAudioIO::GetDefaultOptions(project);
 
    TransportUtilities::PlayPlayRegionAndWait(
       context, SelectedRegion(t0, t1),
@@ -683,7 +540,7 @@ void OnPlayBeforeSelectionStart(const CommandContext &context)
    double beforeLen;
    gPrefs->Read(wxT("/AudioIO/CutPreviewBeforeLen"), &beforeLen, 2.0);
 
-   auto playOptions = DefaultPlayOptions( project );
+   auto playOptions = ProjectAudioIO::GetDefaultOptions(project);
 
    TransportUtilities::PlayPlayRegionAndWait(
       context, SelectedRegion(t0 - beforeLen, t0),
@@ -705,7 +562,7 @@ void OnPlayAfterSelectionStart(const CommandContext &context)
    double afterLen;
    gPrefs->Read(wxT("/AudioIO/CutPreviewAfterLen"), &afterLen, 1.0);
 
-   auto playOptions = DefaultPlayOptions( project );
+   auto playOptions = ProjectAudioIO::GetDefaultOptions(project);
 
    if ( t1 - t0 > 0.0 && t1 - t0 < afterLen )
       TransportUtilities::PlayPlayRegionAndWait(
@@ -732,7 +589,7 @@ void OnPlayBeforeSelectionEnd(const CommandContext &context)
    double beforeLen;
    gPrefs->Read(wxT("/AudioIO/CutPreviewBeforeLen"), &beforeLen, 2.0);
 
-   auto playOptions = DefaultPlayOptions( project );
+   auto playOptions = ProjectAudioIO::GetDefaultOptions(project);
 
    if ( t1 - t0 > 0.0 && t1 - t0 < beforeLen )
       TransportUtilities::PlayPlayRegionAndWait(
@@ -758,7 +615,7 @@ void OnPlayAfterSelectionEnd(const CommandContext &context)
    double afterLen;
    gPrefs->Read(wxT("/AudioIO/CutPreviewAfterLen"), &afterLen, 1.0);
 
-   auto playOptions = DefaultPlayOptions( project );
+   auto playOptions = ProjectAudioIO::GetDefaultOptions(project);
 
    TransportUtilities::PlayPlayRegionAndWait(
       context, SelectedRegion(t1, t1 + afterLen),
@@ -783,7 +640,7 @@ void OnPlayBeforeAndAfterSelectionStart
    double afterLen;
    gPrefs->Read(wxT("/AudioIO/CutPreviewAfterLen"), &afterLen, 1.0);
 
-   auto playOptions = DefaultPlayOptions( project );
+   auto playOptions = ProjectAudioIO::GetDefaultOptions(project);
 
    if ( t1 - t0 > 0.0 && t1 - t0 < afterLen )
       TransportUtilities::PlayPlayRegionAndWait(
@@ -813,7 +670,7 @@ void OnPlayBeforeAndAfterSelectionEnd
    double afterLen;
    gPrefs->Read(wxT("/AudioIO/CutPreviewAfterLen"), &afterLen, 1.0);
 
-   auto playOptions = DefaultPlayOptions( project );
+   auto playOptions = ProjectAudioIO::GetDefaultOptions(project);
 
    if ( t1 - t0 > 0.0 && t1 - t0 < beforeLen )
       TransportUtilities::PlayPlayRegionAndWait(
@@ -836,66 +693,6 @@ void OnPlayCutPreview(const CommandContext &context)
    TransportUtilities::PlayCurrentRegionAndWait(context, false, true);
 }
 
-void OnPlayAtSpeed(const CommandContext &context)
-{
-   auto &project = context.project;
-   auto tb = &TranscriptionToolBar::Get( project );
-
-   if (tb) {
-      tb->PlayAtSpeed(false, false);
-   }
-}
-
-void OnPlayAtSpeedLooped(const CommandContext &context)
-{
-   auto &project = context.project;
-   auto tb = &TranscriptionToolBar::Get( project );
-
-   if (tb) {
-      tb->PlayAtSpeed(true, false);
-   }
-}
-
-void OnPlayAtSpeedCutPreview(const CommandContext &context)
-{
-   auto &project = context.project;
-   auto tb = &TranscriptionToolBar::Get( project );
-
-   if (tb) {
-      tb->PlayAtSpeed(false, true);
-   }
-}
-
-void OnSetPlaySpeed(const CommandContext &context)
-{
-   auto &project = context.project;
-   auto tb = &TranscriptionToolBar::Get( project );
-
-   if (tb) {
-      tb->ShowPlaySpeedDialog();
-   }
-}
-
-void OnPlaySpeedInc(const CommandContext &context)
-{
-   auto &project = context.project;
-   auto tb = &TranscriptionToolBar::Get( project );
-
-   if (tb) {
-      tb->AdjustPlaySpeed(0.1f);
-   }
-}
-
-void OnPlaySpeedDec(const CommandContext &context)
-{
-   auto &project = context.project;
-   auto tb = &TranscriptionToolBar::Get( project );
-
-   if (tb) {
-      tb->AdjustPlaySpeed(-0.1f);
-   }
-}
-
 void OnMoveToPrevLabel(const CommandContext &context)
 {
    auto &project = context.project;
@@ -926,51 +723,34 @@ void OnStopSelect(const CommandContext &context)
 }
 #endif
 
-}; // struct Handler
-
-} // namespace
-
-static CommandHandlerObject &findCommandHandler(AudacityProject &) {
-   // Handler is not stateful.  Doesn't need a factory registered with
-   // AudacityProject.
-   static TransportActions::Handler instance;
-   return instance;
-};
-
 // Menu definitions
 
-#define FN(X) (& TransportActions::Handler :: X)
-
 // Under /MenuBar
-namespace {
-using namespace MenuTable;
-BaseItemSharedPtr TransportMenu()
+using namespace MenuRegistry;
+auto TransportMenu()
 {
-   using Options = CommandManager::Options;
-
-   static const auto CanStopFlags = AudioIONotBusyFlag() | CanStopAudioStreamFlag();
-
-   static BaseItemSharedPtr menu{
-   ( FinderScope{ findCommandHandler },
+   static const auto CanStopFlags =
+      AudioIONotBusyFlag() | CanStopAudioStreamFlag();
+   static auto menu = std::shared_ptr{
    /* i18n-hint: 'Transport' is the name given to the set of controls that
       play, record, pause etc. */
    Menu( wxT("Transport"), XXO("Tra&nsport"),
       Section( "Basic",
          Menu( wxT("Play"), XXO("Pl&aying"),
             /* i18n-hint: (verb) Start or Stop audio playback*/
-            Command( wxT("DefaultPlayStop"), XXO("Pl&ay/Stop"), FN(OnPlayDefaultOrStop),
+            Command( wxT("DefaultPlayStop"), XXO("Pl&ay/Stop"), OnPlayDefaultOrStop,
                CanStopAudioStreamFlag(), wxT("Space") ),
             Command( wxT("PlayStopSelect"), XXO("Play/Stop and &Set Cursor"),
-               FN(OnPlayStopSelect), CanStopAudioStreamFlag(), wxT("X") ),
-            Command( wxT("OncePlayStop"), XXO("Play &Once/Stop"), FN(OnPlayOnceOrStop),
+               OnPlayStopSelect, CanStopAudioStreamFlag(), wxT("X") ),
+            Command( wxT("OncePlayStop"), XXO("Play &Once/Stop"), OnPlayOnceOrStop,
                CanStopAudioStreamFlag(), wxT("Shift+Space") ),
-            Command( wxT("Pause"), XXO("&Pause"), FN(OnPause),
+            Command( wxT("Pause"), XXO("&Pause"), OnPause,
                CanStopAudioStreamFlag(), wxT("P") )
          ),
 
          Menu( wxT("Record"), XXO("&Recording"),
             /* i18n-hint: (verb)*/
-            Command( wxT("Record1stChoice"), XXO("&Record"), FN(OnRecord),
+            Command( wxT("Record1stChoice"), XXO("&Record"), OnRecord,
                CanStopFlags, wxT("R") ),
 
             // The OnRecord2ndChoice function is: if normal record records beside,
@@ -985,17 +765,13 @@ BaseItemSharedPtr TransportMenu()
                // It should be bound to Shift+R
                (gPrefs->ReadBool("/GUI/PreferNewTrackRecord", false)
                 ? XXO("&Append Record") : XXO("Record &New Track")),
-               FN(OnRecord2ndChoice), CanStopFlags,
-               wxT("Shift+R"),
-               findCommandHandler
+               OnRecord2ndChoice, CanStopFlags,
+               wxT("Shift+R")
             ); },
 
-            Command( wxT("TimerRecord"), XXO("&Timer Record..."),
-               FN(OnTimerRecord), CanStopFlags, wxT("Shift+T") ),
-
    #ifdef EXPERIMENTAL_PUNCH_AND_ROLL
             Command( wxT("PunchAndRoll"), XXO("Punch and Rol&l Record"),
-               FN(OnPunchAndRoll),
+               OnPunchAndRoll,
                WaveTracksExistFlag() | AudioIONotBusyFlag(), wxT("Shift+D") ),
    #endif
 
@@ -1003,7 +779,7 @@ BaseItemSharedPtr TransportMenu()
             // rather than put it at the top level.
             // CommandManger::AddItem can now cope with simple duplicated items.
             // PRL:  caution, this is a duplicated command name!
-            Command( wxT("Pause"), XXO("&Pause"), FN(OnPause),
+            Command( wxT("Pause"), XXO("&Pause"), OnPause,
                CanStopAudioStreamFlag(), wxT("P") )
          )
       ),
@@ -1012,62 +788,48 @@ BaseItemSharedPtr TransportMenu()
          Section( "",
             Menu( wxT("PlayRegion"), XXO("&Looping"),
                Command( wxT("TogglePlayRegion"), LoopToggleText,
-                  FN(OnTogglePlayRegion), AlwaysEnabledFlag,
+                  OnTogglePlayRegion, AlwaysEnabledFlag,
                      Options(L"L").CheckTest([](const AudacityProject& project){
                          return IsLoopingEnabled(project);
                      } )),
                Command( wxT("ClearPlayRegion"), XXO("&Clear Loop"),
-                  FN(OnClearPlayRegion), AlwaysEnabledFlag, L"Shift+Alt+L" ),
+                  OnClearPlayRegion, AlwaysEnabledFlag, L"Shift+Alt+L" ),
                Command( wxT("SetPlayRegionToSelection"),
                   XXO("&Set Loop to Selection"),
-                  FN(OnSetPlayRegionToSelection), AlwaysEnabledFlag,
+                  OnSetPlayRegionToSelection, AlwaysEnabledFlag,
                      L"Shift+L" ),
                Command( wxT("SetPlayRegionIn"),
                   SetLoopInTitle,
-                  FN(OnSetPlayRegionIn), AlwaysEnabledFlag ),
+                  OnSetPlayRegionIn, AlwaysEnabledFlag ),
                Command( wxT("SetPlayRegionOut"),
                   SetLoopOutTitle,
-                  FN(OnSetPlayRegionOut), AlwaysEnabledFlag )
+                  OnSetPlayRegionOut, AlwaysEnabledFlag )
             )
          ),
 
          Command( wxT("RescanDevices"), XXO("R&escan Audio Devices"),
-            FN(OnRescanDevices), AudioIONotBusyFlag() | CanStopAudioStreamFlag() ),
+            OnRescanDevices, AudioIONotBusyFlag() | CanStopAudioStreamFlag() ),
 
          Menu( wxT("Options"), XXO("Transport &Options"),
-            Section( "",
+            Section( "Part1",
                // Sound Activated recording options
                Command( wxT("SoundActivationLevel"),
-                  XXO("Sound Activation Le&vel..."), FN(OnSoundActivated),
+                  XXO("Set sound activation le&vel..."), OnSoundActivated,
                   AudioIONotBusyFlag() | CanStopAudioStreamFlag() ),
                Command( wxT("SoundActivation"),
-                  XXO("Sound A&ctivated Recording (on/off)"),
-                  FN(OnToggleSoundActivated),
+                  XXO("Enable sound a&ctivated recording"),
+                  OnToggleSoundActivated,
                   AudioIONotBusyFlag() | CanStopAudioStreamFlag(),
-                  Options{}.CheckTest(wxT("/AudioIO/SoundActivatedRecord"), false) )
+                  Options{}.CheckTest(SoundActivatedRecord) )
             ),
 
-            Section( "",
-               Command( wxT("PinnedHead"), XXO("Pinned Play/Record &Head (on/off)"),
-                  FN(OnTogglePinnedHead),
-                  // Switching of scrolling on and off is permitted
-                  // even during transport
-                  AlwaysEnabledFlag,
-                  Options{}.CheckTest([](const AudacityProject&){
-                     return TracksPrefs::GetPinnedHeadPreference(); } ) ),
-
-               Command( wxT("Overdub"), XXO("&Overdub (on/off)"),
-                  FN(OnTogglePlayRecording),
+            Section( "Part2",
+               Command( wxT("Overdub"), XXO("Hear &other tracks during recording"),
+                  OnTogglePlayRecording,
                   AudioIONotBusyFlag() | CanStopAudioStreamFlag(),
-                  Options{}.CheckTest( wxT("/AudioIO/Duplex"),
-#ifdef EXPERIMENTAL_DA
-                     false
-#else
-                     true
-#endif
-                  ) ),
-               Command( wxT("SWPlaythrough"), XXO("So&ftware Playthrough (on/off)"),
-                  FN(OnToggleSWPlaythrough),
+                  Options{}.CheckTest( wxT("/AudioIO/Duplex"), true) ),
+               Command( wxT("SWPlaythrough"), XXO("Enable audible input &monitoring"),
+                  OnToggleSWPlaythrough,
                   AudioIONotBusyFlag() | CanStopAudioStreamFlag(),
                   Options{}.CheckTest( wxT("/AudioIO/SWPlaythrough"), false ) )
 
@@ -1076,7 +838,7 @@ BaseItemSharedPtr TransportMenu()
                ,
                Command( wxT("AutomatedInputLevelAdjustmentOnOff"),
                   XXO("A&utomated Recording Level Adjustment (on/off)"),
-                  FN(OnToggleAutomatedInputLevelAdjustment),
+                  OnToggleAutomatedInputLevelAdjustment,
                   AudioIONotBusyFlag() | CanStopAudioStreamFlag(),
                   Options{}.CheckTest(
                      wxT("/AudioIO/AutomatedInputLevelAdjustment"), false ) )
@@ -1084,112 +846,75 @@ BaseItemSharedPtr TransportMenu()
             )
          )
       )
-   ) ) };
+   ) };
    return menu;
 }
 
-AttachedItem sAttachment1{
-   wxT(""),
-   Shared( TransportMenu() )
-};
+AttachedItem sAttachment1{ Indirect(TransportMenu()) };
 
-BaseItemSharedPtr ExtraTransportMenu()
+auto ExtraTransportMenu()
 {
-   static BaseItemSharedPtr menu{
-   ( FinderScope{ findCommandHandler },
+   static auto menu = std::shared_ptr{
    Menu( wxT("Transport"), XXO("T&ransport"),
       // PlayStop is already in the menus.
       /* i18n-hint: (verb) Start playing audio*/
-      Command( wxT("Play"), XXO("Pl&ay Once"), FN(OnPlayOnceOrStop),
+      Command( wxT("Play"), XXO("Pl&ay Once"), OnPlayOnceOrStop,
          WaveTracksExistFlag() | AudioIONotBusyFlag() ),
       /* i18n-hint: (verb) Stop playing audio*/
-      Command( wxT("Stop"), XXO("Sto&p"), FN(OnStop),
+      Command( wxT("Stop"), XXO("Sto&p"), OnStop,
          AudioIOBusyFlag() | CanStopAudioStreamFlag() ),
-      Command( wxT("PlayOneSec"), XXO("Play &One Second"), FN(OnPlayOneSecond),
+      Command( wxT("PlayOneSec"), XXO("Play &One Second"), OnPlayOneSecond,
          CaptureNotBusyFlag(), wxT("1") ),
       Command( wxT("PlayToSelection"), XXO("Play to &Selection"),
-         FN(OnPlayToSelection),
+         OnPlayToSelection,
          CaptureNotBusyFlag(), wxT("B") ),
       Command( wxT("PlayBeforeSelectionStart"),
-         XXO("Play &Before Selection Start"), FN(OnPlayBeforeSelectionStart),
+         XXO("Play &Before Selection Start"), OnPlayBeforeSelectionStart,
          CaptureNotBusyFlag(), wxT("Shift+F5") ),
       Command( wxT("PlayAfterSelectionStart"),
-         XXO("Play Af&ter Selection Start"), FN(OnPlayAfterSelectionStart),
+         XXO("Play Af&ter Selection Start"), OnPlayAfterSelectionStart,
          CaptureNotBusyFlag(), wxT("Shift+F6") ),
       Command( wxT("PlayBeforeSelectionEnd"),
-         XXO("Play Be&fore Selection End"), FN(OnPlayBeforeSelectionEnd),
+         XXO("Play Be&fore Selection End"), OnPlayBeforeSelectionEnd,
          CaptureNotBusyFlag(), wxT("Shift+F7") ),
       Command( wxT("PlayAfterSelectionEnd"),
-         XXO("Play Aft&er Selection End"), FN(OnPlayAfterSelectionEnd),
+         XXO("Play Aft&er Selection End"), OnPlayAfterSelectionEnd,
          CaptureNotBusyFlag(), wxT("Shift+F8") ),
       Command( wxT("PlayBeforeAndAfterSelectionStart"),
          XXO("Play Before a&nd After Selection Start"),
-         FN(OnPlayBeforeAndAfterSelectionStart), CaptureNotBusyFlag(),
+         OnPlayBeforeAndAfterSelectionStart, CaptureNotBusyFlag(),
          wxT("Ctrl+Shift+F5") ),
       Command( wxT("PlayBeforeAndAfterSelectionEnd"),
          XXO("Play Before an&d After Selection End"),
-         FN(OnPlayBeforeAndAfterSelectionEnd), CaptureNotBusyFlag(),
+         OnPlayBeforeAndAfterSelectionEnd, CaptureNotBusyFlag(),
          wxT("Ctrl+Shift+F7") ),
       Command( wxT("PlayCutPreview"), XXO("Play C&ut Preview"),
-         FN(OnPlayCutPreview),
+         OnPlayCutPreview,
          CaptureNotBusyFlag(), wxT("C") )
-   ) ) };
+   ) };
    return menu;
 }
 
-AttachedItem sAttachment2{
-   wxT("Optional/Extra/Part1"),
-   Shared( ExtraTransportMenu() )
+AttachedItem sAttachment2{ Indirect(ExtraTransportMenu()),
+   wxT("Optional/Extra/Part1")
 };
 
-BaseItemSharedPtr ExtraPlayAtSpeedMenu()
+auto ExtraSelectionItems()
 {
-   static BaseItemSharedPtr menu{
-   ( FinderScope{ findCommandHandler },
-   Menu( wxT("PlayAtSpeed"), XXO("&Play-at-Speed"),
-      /* i18n-hint: 'Normal Play-at-Speed' doesn't loop or cut preview. */
-      Command( wxT("PlayAtSpeedLooped"), XXO("&Play-at-Speed"),
-         FN(OnPlayAtSpeedLooped), CaptureNotBusyFlag() ),
-      Command( wxT("PlayAtSpeed"), XXO("Play-at-Speed &Once"),
-         FN(OnPlayAtSpeed), CaptureNotBusyFlag() ),
-      Command( wxT("PlayAtSpeedCutPreview"), XXO("Play C&ut Preview-at-Speed"),
-         FN(OnPlayAtSpeedCutPreview), CaptureNotBusyFlag() ),
-      Command( wxT("SetPlaySpeed"), XXO("Ad&just Playback Speed..."),
-         FN(OnSetPlaySpeed), CaptureNotBusyFlag() ),
-      Command( wxT("PlaySpeedInc"), XXO("&Increase Playback Speed"),
-         FN(OnPlaySpeedInc), CaptureNotBusyFlag() ),
-      Command( wxT("PlaySpeedDec"), XXO("&Decrease Playback Speed"),
-         FN(OnPlaySpeedDec), CaptureNotBusyFlag() )
-   ) ) };
-   return menu;
-}
-
-AttachedItem sAttachment3{
-   wxT("Optional/Extra/Part1"),
-   Shared( ExtraPlayAtSpeedMenu() )
-};
-
-BaseItemSharedPtr ExtraSelectionItems()
-{
-   using Options = CommandManager::Options;
-   static BaseItemSharedPtr items{
-   (FinderScope{ findCommandHandler },
+   static auto items = std::shared_ptr{
    Items(wxT("MoveToLabel"),
       Command(wxT("MoveToPrevLabel"), XXO("Move to Pre&vious Label"),
-         FN(OnMoveToPrevLabel),
+         OnMoveToPrevLabel,
          CaptureNotBusyFlag() | TrackPanelHasFocus(), wxT("Alt+Left")),
       Command(wxT("MoveToNextLabel"), XXO("Move to Ne&xt Label"),
-         FN(OnMoveToNextLabel),
+         OnMoveToNextLabel,
          CaptureNotBusyFlag() | TrackPanelHasFocus(), wxT("Alt+Right"))
-   )) };
+   ) };
    return items;
 }
 
-AttachedItem sAttachment4{
-  { wxT("Optional/Extra/Part1/Select"), { OrderingHint::End, {} } },
-  Shared(ExtraSelectionItems())
+AttachedItem sAttachment4{ Indirect(ExtraSelectionItems()),
+  { wxT("Optional/Extra/Part1/Select"), { OrderingHint::End, {} } }
 };
 
 }
-
-#undef FN
