@@ -47,7 +47,7 @@
 
 #ifdef EXPERIMENTAL_VOICE_DETECTION
 #include "../VoiceKey.h"
-#include "../ProjectWindow.h"
+
 #endif
 
 IMPLEMENT_CLASS(TranscriptionToolBar, ToolBar);
@@ -112,13 +112,7 @@ TranscriptionToolBar::~TranscriptionToolBar()
 
 bool TranscriptionToolBar::ShownByDefault() const
 {
-   return
-#ifdef EXPERIMENTAL_DA
-      false
-#else
-      true
-#endif
-   ;
+   return true;
 }
 
 ToolBar::DockID TranscriptionToolBar::DefaultDockID() const
@@ -240,8 +234,8 @@ void TranscriptionToolBar::Populate()
       ASlider::Options{}
          .Style( SPEED_SLIDER )
          //  6 steps using page up/down, and 60 using arrow keys
-         .Line( 0.16667f )
-         .Page( 1.6667f )
+         .Line( 0.16722f )
+         .Page( 1.6722f )
    );
    mPlaySpeedSlider->SetSizeHints(wxSize(100, 25), wxSize(2000, 25));
    mPlaySpeedSlider->Set(mPlaySpeed / 100.0);
@@ -616,7 +610,7 @@ void TranscriptionToolBar::OnStartOn(wxCommandEvent & WXUNUSED(event))
 
       auto &selectedRegion = ViewInfo::Get( mProject ).selectedRegion;
       selectedRegion.setT0( newpos );
-      ProjectWindow::Get( mProject ).RedrawProject();
+      Viewport::Get(mProject).Redraw();
 
       SetButton(false, mButtons[TTB_StartOn]);
    }
@@ -649,7 +643,7 @@ void TranscriptionToolBar::OnStartOff(wxCommandEvent & WXUNUSED(event))
 
       auto &selectedRegion = ViewInfo::Get( mProject ).selectedRegion;
       selectedRegion.setT0( newpos );
-      ProjectWindow::Get( mProject ).RedrawProject();
+      Viewport::Get(mProject).Redraw();
 
       SetButton(false, mButtons[TTB_StartOn]);
    }
@@ -684,7 +678,7 @@ void TranscriptionToolBar::OnEndOn(wxCommandEvent & WXUNUSED(event))
 
       auto &selectedRegion = ViewInfo::Get( mProject ).selectedRegion;
       selectedRegion.setT1( newpos );
-      ProjectWindow::Get( mProject ).RedrawProject();
+      Viewport::Get(mProject).Redraw();
 
       SetButton(false, mButtons[TTB_EndOn]);
    }
@@ -720,7 +714,7 @@ void TranscriptionToolBar::OnEndOff(wxCommandEvent & WXUNUSED(event))
 
       auto &selectedRegion = ViewInfo::Get( mProject ).selectedRegion;
       selectedRegion.setT1( newpos );
-      ProjectWindow::Get( mProject ).RedrawProject();
+      Viewport::Get(mProject).Redraw();
 
       SetButton(false, mButtons[TTB_EndOff]);
    }
@@ -760,7 +754,7 @@ void TranscriptionToolBar::OnSelectSound(wxCommandEvent & WXUNUSED(event))
       auto &selectedRegion = ViewInfo::Get( mProject ).selectedRegion;
       selectedRegion.setTimes(
          newstart.as_double() / rate, newend.as_double() /  rate );
-      ProjectWindow::Get( mProject ).RedrawProject();
+      Viewport::Get(mProject).Redraw();
 
    }
 
@@ -797,7 +791,7 @@ void TranscriptionToolBar::OnSelectSilence(wxCommandEvent & WXUNUSED(event))
       auto &selectedRegion = ViewInfo::Get( mProject ).selectedRegion;
       selectedRegion.setTimes(
          newstart.as_double() / rate, newend.as_double() / rate);
-      ProjectWindow::Get( mProject ).RedrawProject();
+      Viewport::Get(mProject).Redraw();
 
    }
 
@@ -851,7 +845,7 @@ void TranscriptionToolBar::OnCalibrate(wxCommandEvent & WXUNUSED(event))
 #include "../LabelTrack.h"
 #include "ProjectHistory.h"
 #include "../TrackPanel.h"
-#include "../TrackPanelAx.h"
+#include "TrackFocus.h"
 #include "../tracks/labeltrack/ui/LabelTrackView.h"
 namespace {
 int DoAddLabel(
@@ -979,7 +973,7 @@ void TranscriptionToolBar::OnAutomateSelection(wxCommandEvent & WXUNUSED(event))
          start = newEnd;
 
          DoAddLabel(mProject, SelectedRegion(newStartPos, newEndPos));
-      ProjectWindow::Get( mProject ).RedrawProject();
+      Viewport::Get(mProject).Redraw();
       }
       SetButton(false, mButtons[TTB_AutomateSelection]);
    }
@@ -1146,23 +1140,22 @@ void OnPlaySpeedDec(const CommandContext &context)
    }
 }
 
-using namespace MenuTable;
+using namespace MenuRegistry;
 
-BaseItemSharedPtr ExtraPlayAtSpeedMenu()
+auto ExtraPlayAtSpeedMenu()
 {
-   static BaseItemSharedPtr menu{
+   static auto menu = std::shared_ptr{
       Menu( wxT("PlayAtSpeed"), XXO("&Play-at-Speed") ) };
    return menu;
 }
 
-AttachedItem sAttachment2{
-   wxT("Optional/Extra/Part1"),
-   Indirect(ExtraPlayAtSpeedMenu())
+AttachedItem sAttachment2{ Indirect(ExtraPlayAtSpeedMenu()),
+   wxT("Optional/Extra/Part1")
 };
 
-BaseItemSharedPtr ExtraPlayAtSpeedItems()
+auto ExtraPlayAtSpeedItems()
 {
-   static BaseItemSharedPtr items{
+   static auto items = std::shared_ptr{
    Items( "",
       /* i18n-hint: 'Normal Play-at-Speed' doesn't loop or cut preview. */
       Command( wxT("PlayAtSpeedLooped"), XXO("&Play-at-Speed"),
@@ -1181,10 +1174,9 @@ BaseItemSharedPtr ExtraPlayAtSpeedItems()
    return items;
 }
 
-AttachedItem sAttachment3{
+AttachedItem sAttachment3{ Indirect(ExtraPlayAtSpeedItems()),
    Placement{ wxT("Optional/Extra/Part1/PlayAtSpeed"),
-     OrderingHint::Begin },
-   Indirect(ExtraPlayAtSpeedItems())
+     OrderingHint::Begin }
 };
 
 }
