@@ -17,15 +17,14 @@ Paul Licameli split from TrackMenus.cpp
 // Attach a menu item for diagnostic information
 
 #include "AudioIOBase.h"
-#include "../../../../commands/CommandContext.h"
-#include "../../../../commands/CommandManager.h"
+#include "CommandContext.h"
+#include "MenuRegistry.h"
 #include "CommonCommandFlags.h"
 #include "HelpUtilities.h"
 #include "NoteTrack.h"
 
 #ifdef EXPERIMENTAL_MIDI_OUT
 namespace {
-struct Handler : CommandHandlerObject {
 void OnMidiDeviceInfo(const CommandContext &context)
 {
    auto &project = context.project;
@@ -34,26 +33,13 @@ void OnMidiDeviceInfo(const CommandContext &context)
    ShowDiagnostics( project, info,
       XO("MIDI Device Info"), wxT("midideviceinfo.txt") );
 }
-};
 
-static CommandHandlerObject &findCommandHandler(AudacityProject &) {
-   // Handler is not stateful.  Doesn't need a factory registered with
-   // AudacityProject.
-   static Handler instance;
-   return instance;
-};
-
-using namespace MenuTable;
-#define FN(X) (& Handler :: X)
+using namespace MenuRegistry;
 AttachedItem sAttachment{
+   Command( wxT("MidiDeviceInfo"), XXO("&MIDI Device Info..."),
+      OnMidiDeviceInfo, AudioIONotBusyFlag() ),
    { wxT("Help/Other/Diagnostics"),
-      { OrderingHint::After, wxT("DeviceInfo") } },
-   ( FinderScope{ findCommandHandler },
-      Command( wxT("MidiDeviceInfo"), XXO("&MIDI Device Info..."),
-         FN(OnMidiDeviceInfo),
-         AudioIONotBusyFlag() )
-   )
+      { OrderingHint::After, wxT("DeviceInfo") } }
 };
-#undef FN
 }
 #endif
