@@ -19,6 +19,7 @@ class wxMouseState;
 
 class Track;
 class ViewInfo;
+class WaveClip;
 class WaveTrack;
 
 class SampleHandle final : public UIHandle
@@ -42,7 +43,7 @@ public:
 
    virtual ~SampleHandle();
 
-   std::shared_ptr<WaveTrack> GetTrack() const { return mClickedTrack; }
+   std::shared_ptr<const Channel> FindChannel() const override;
 
    void Enter(bool forward, AudacityProject *) override;
 
@@ -69,10 +70,11 @@ private:
       (const wxMouseEvent &event, const ViewInfo &viewInfo, double t0);
 
    std::shared_ptr<WaveTrack> mClickedTrack;
+   WaveClip* mClickedClip {};
    wxRect mRect{};
 
-   sampleCount mClickedStartSample{};
-   sampleCount mLastDragSample{};
+   int mClickedStartPixel {};
+   int mLastDragPixel {};
    float mLastDragSampleValue{};
    bool mAltKey{};
 };
