@@ -14,8 +14,10 @@
 
 #include <wx/event.h>
 
+#include "ClipParameters.h"
+#include "../../../../TrackArt.h"
 #include "../../../../TrackArtist.h"
-#include "../../../../Snap.h"
+#include "Snap.h"
 #include "../../../../TrackPanelDrawingContext.h"
 #include "../../../../../images/Cursors.h"
 #include "WaveClip.h"
@@ -62,6 +64,8 @@ namespace {
     virtual void Finish(AudacityProject& project) = 0;
     virtual void Cancel() = 0;
 
+    virtual std::shared_ptr<Track> FindTrack() const = 0;
+
     virtual void Draw(
         TrackPanelDrawingContext &context,
         const wxRect &rect,
@@ -83,16 +87,19 @@ double GetLeftAdjustLimit(const WaveTrack::Interval& interval,
                           bool isStretchMode)
 {
    if (!adjustingLeftBorder)
-      return interval.Start() + 1.0 / track.GetRate();
+      return std::min(
+         interval.GetSequenceEndTime(),
+         interval.Start() + 1.0 / track.GetRate()
+      );
 
    const auto prevInterval = track.GetNextInterval(interval, PlaybackDirection::backward);
    if(isStretchMode)
       return prevInterval ? prevInterval->End() :
                             std::numeric_limits<double>::lowest();
    if(prevInterval)
-      return std::max(interval.GetClip(0)->GetSequenceStartTime(),
+      return std::max(interval.GetSequenceStartTime(),
                 prevInterval->End());
-   return interval.GetClip(0)->GetSequenceStartTime();
+   return interval.GetSequenceStartTime();
 }
 
 double GetRightAdjustLimit(
@@ -100,7 +107,10 @@ double GetRightAdjustLimit(
    bool isStretchMode)
 {
    if (adjustingLeftBorder)
-      return interval.End() - 1.0 / track.GetRate();
+      return std::max(
+         interval.GetSequenceStartTime(),
+         interval.End() - 1.0 / track.GetRate()
+      );
 
    const auto nextInterval = track.GetNextInterval(interval, PlaybackDirection::forward);
    if (isStretchMode)
@@ -108,9 +118,9 @@ double GetRightAdjustLimit(
                             std::numeric_limits<double>::max();
 
    if(nextInterval)
-      return std::min(interval.GetClip(0)->GetSequenceEndTime(),
+      return std::min(interval.GetSequenceEndTime(),
                       nextInterval->Start());
-   return interval.GetClip(0)->GetSequenceEndTime();
+   return interval.GetSequenceEndTime();
 }
 } // namespace
 
@@ -192,7 +202,7 @@ public:
       , mIsStretchMode { isStretchMode }
       , mInitialBorderPosition { adjustLeftBorder ? mInterval->Start() :
                                              mInterval->End() }
-      , mBorderPosition { mInitialBorderPosition } 
+      , mBorderPosition { mInitialBorderPosition }
       , mRange { GetLeftAdjustLimit( *mInterval, *mTrack, adjustLeftBorder, isStretchMode),
                  GetRightAdjustLimit(*mInterval, *mTrack, adjustLeftBorder, isStretchMode) }
       , mAdjustHandler { std::move(adjustHandler) }
@@ -207,6 +217,11 @@ public:
       }
    }
 
+   std::shared_ptr<Track> FindTrack() const override
+   {
+      return mTrack;
+   }
+
    bool Init(const TrackPanelMouseEvent& event) override
    {
       if (event.event.LeftDown())
@@ -223,7 +238,7 @@ public:
       const auto dx = eventX - mDragStartX;
 
       const auto& viewInfo = ViewInfo::Get(project);
-      
+
       const auto eventT = viewInfo.PositionToTime(
          viewInfo.TimeToPosition(mInitialBorderPosition, event.rect.x) + dx,
          event.rect.x
@@ -258,7 +273,7 @@ public:
       {
          if (mIsStretchMode)
          {
-            PushClipSpeedChangedUndoState(
+            WaveClipUtilities::PushClipSpeedChangedUndoState(
                project, 100.0 / mInterval->GetStretchRatio());
          }
          else if (mAdjustingLeftBorder)
@@ -288,7 +303,7 @@ public:
       if(iPass == TrackArtist::PassSnapping && mSnap.Snapped())
       {
          auto &dc = context.dc;
-         SnapManager::Draw(&dc, rect.x + mSnap.outCoord, -1);
+         TrackArt::DrawSnapLines(&dc, rect.x + mSnap.outCoord, -1);
       }
    }
 
@@ -336,10 +351,13 @@ HitTestPreview WaveClipAdjustBorderHandle::HitPreviewStretch(const AudacityProje
    };
 }
 
-WaveClipAdjustBorderHandle::WaveClipAdjustBorderHandle(std::unique_ptr<AdjustPolicy>& adjustPolicy,
-                                                       bool stretchMode,
-                                                       bool leftBorder)
+WaveClipAdjustBorderHandle::WaveClipAdjustBorderHandle(
+   std::unique_ptr<AdjustPolicy>& adjustPolicy,
+   std::shared_ptr<const WaveTrack> pTrack,
+   bool stretchMode,
+   bool leftBorder)
    : mAdjustPolicy{ std::move(adjustPolicy) }
+   , mpTrack{ move(pTrack) }
    , mIsStretchMode{stretchMode}
    , mIsLeftBorder{leftBorder}
 {
@@ -348,6 +366,11 @@ WaveClipAdjustBorderHandle::WaveClipAdjustBorderHandle(std::unique_ptr<AdjustPol
 
 WaveClipAdjustBorderHandle::~WaveClipAdjustBorderHandle() = default;
 
+std::shared_ptr<const Channel> WaveClipAdjustBorderHandle::FindChannel() const
+{
+   return mpTrack;
+}
+
 WaveClipAdjustBorderHandle::WaveClipAdjustBorderHandle(WaveClipAdjustBorderHandle&&) noexcept = default;
 
 WaveClipAdjustBorderHandle& WaveClipAdjustBorderHandle::operator=(WaveClipAdjustBorderHandle&&) noexcept = default;
@@ -440,7 +463,7 @@ UIHandlePtr WaveClipAdjustBorderHandle::HitAnywhere(
 
       return AssignUIHandlePtr(
          holder,
-         std::make_shared<WaveClipAdjustBorderHandle>(policy,
+         std::make_shared<WaveClipAdjustBorderHandle>(policy, waveTrack,
             isStretchMode,
             adjustLeftBorder));
    }
