@@ -9,31 +9,33 @@ Paul Licameli split from TrackPanel.cpp
 **********************************************************************/
 
 #include "TimeTrackView.h"
-#include "../../../TimeTrack.h"
-
-#include "../../../Experimental.h"
+#include "TimeTrack.h"
 
 #include "TimeTrackControls.h"
 
 #include "TimeTrackVRulerControls.h"
-#include "../../../AColor.h"
-#include "../../../AllThemeResources.h"
-#include "../../../Envelope.h"
+#include "AColor.h"
+#include "AllThemeResources.h"
+#include "Envelope.h"
 #include "../../../EnvelopeEditor.h"
 #include "../../../HitTestResult.h"
-#include "../../../Theme.h"
+#include "Theme.h"
 #include "../../../TrackArtist.h"
 #include "../../../TrackPanelDrawingContext.h"
 #include "../../../TrackPanelMouseEvent.h"
-#include "../../../ViewInfo.h"
+#include "ViewInfo.h"
 #include "../../../widgets/Ruler.h"
+#include "../../../widgets/LinearUpdater.h"
+#include "../../../widgets/TimeFormat.h"
 
 #include "../../ui/EnvelopeHandle.h"
 
 #include <wx/dc.h>
 
-TimeTrackView::TimeTrackView( const std::shared_ptr<Track> &pTrack )
-   : CommonTrackView{ pTrack }
+using Doubles = ArrayOf<double>;
+
+TimeTrackView::TimeTrackView(const std::shared_ptr<Track> &pTrack)
+   : CommonChannelView{ pTrack, 0 }
 {
 }
 
@@ -54,18 +56,17 @@ std::vector<UIHandlePtr> TimeTrackView::DetailedHitTest
    return results;
 }
 
-using DoGetTimeTrackView = DoGetView::Override< TimeTrack >;
-template<> template<> auto DoGetTimeTrackView::Implementation() -> Function {
-   return [](TimeTrack &track) {
-      return std::make_shared<TimeTrackView>( track.SharedPointer() );
+using DoGetTimeTrackView = DoGetView::Override<TimeTrack>;
+DEFINE_ATTACHED_VIRTUAL_OVERRIDE(DoGetTimeTrackView) {
+   return [](TimeTrack &track, size_t) {
+      return std::make_shared<TimeTrackView>(track.SharedPointer());
    };
 }
-static DoGetTimeTrackView registerDoGetTimeTrackView;
 
-std::shared_ptr<TrackVRulerControls> TimeTrackView::DoGetVRulerControls()
+std::shared_ptr<ChannelVRulerControls> TimeTrackView::DoGetVRulerControls()
 {
    return
-      std::make_shared<TimeTrackVRulerControls>( shared_from_this() );
+      std::make_shared<TimeTrackVRulerControls>(shared_from_this());
 }
 
 namespace {
@@ -80,7 +81,7 @@ void DrawHorzRulerAndCurve
    bool highlight = false;
 #ifdef EXPERIMENTAL_TRACK_PANEL_HIGHLIGHTING
    auto target = dynamic_cast<EnvelopeHandle*>(context.target.get());
-   highlight = target && target->GetEnvelope() == this->GetEnvelope();
+   highlight = target && target->GetEnvelope() == track.GetEnvelope();
 #endif
    
    double min = zoomInfo.PositionToTime(0);
@@ -106,8 +107,8 @@ void DrawHorzRulerAndCurve
    ruler.Draw(dc, track.GetEnvelope());
    
    Doubles envValues{ size_t(mid.width) };
-   Envelope::GetValues( *track.GetEnvelope(),
-    0, 0, envValues.get(), mid.width, 0, zoomInfo );
+   CommonChannelView::GetEnvelopeValues(*track.GetEnvelope(),
+      0, 0, envValues.get(), mid.width, 0, zoomInfo);
    
    wxPen &pen = highlight ? AColor::uglyPen : AColor::envelopePen;
    dc.SetPen( pen );
@@ -157,9 +158,33 @@ void TimeTrackView::Draw(
    const wxRect &rect, unsigned iPass )
 {
    if ( iPass == TrackArtist::PassTracks ) {
+      const auto pTrack = FindTrack();
+      const auto pList = pTrack->GetOwner();
+      if (!pList)
+         // Track isn't owned by a list.  Can't proceed!
+         return;
+      const auto pProject = pList->GetOwner();
+      if (!pProject)
+         // List isn't owned by a project.  Can't proceed!
+         // But this shouldn't happen when drawing it
+         return;
+
+      auto &zoomInfo = ViewInfo::Get(*pProject);
+      LinearUpdater updater;
+      updater.SetData(&zoomInfo);
+
+      // Just using a stack-local Ruler object.  Observe the "Invalidate"
+      // call above which has long been done with every redraw.
+      // Avoiding invalidation with every draw, and making the ruler persistent
+      // between drawings, would require that it be invalidated whenever the
+      // user drags points, or the time selection changes -- really too much
+      // work.
+      Ruler ruler{ updater, TimeFormat::Instance() };
+      ruler.SetLabelEdges(false);
+
       const auto tt = std::static_pointer_cast<const TimeTrack>(
-         FindTrack()->SubstitutePendingChangedTrack());
-      DrawTimeTrack( context, *tt, tt->GetRuler(), rect );
+         pTrack->SubstitutePendingChangedTrack());
+      DrawTimeTrack(context, *tt, ruler, rect);
    }
-   CommonTrackView::Draw( context, rect, iPass );
+   CommonChannelView::Draw(context, rect, iPass);
 }
